# -*- coding: utf-8 -*-
"""
utils
"""

from datetime import datetime
import feedparser
from time import mktime

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied

from coop_cms.apps.rss_sync.models import RssSource, RssItem
from coop_cms.models import BaseArticle
from coop_cms.settings import get_article_class


def collect_rss_items(user, source, check_user_rights=True):
    """
    download a rss feed and create rss_items
    the source can be a RssSource or any object with a get_absolute_url method
    """
    if check_user_rights and (not (user.is_staff and user.has_perm('rss_sync.add_rssitem'))):
        raise PermissionDenied

    feed = feedparser.parse(source.get_absolute_url())
    for entry in feed.entries:
        # create RSS entries if not exists
        item = RssItem.objects.get_or_create(link=entry.link, source=source)[0]
        # In any case, update the data
        item.title = entry.title
        try:
            item.updated = datetime.fromtimestamp(mktime(entry.updated_parsed))
        except AttributeError:
            item.updated = datetime.now()

        item.author = getattr(entry, 'author', '')[:100]
        item.summary = entry.summary
        item.save()

    if isinstance(source, RssSource):
        # update info for rss sources only
        source.title = getattr(feed.feed, 'title', '')
        source.last_collect = datetime.now()
        source.save()


def collect_all_rss_items():
    """collect all items"""
    all_models = getattr(settings, 'RSS_SYNC_SOURCE_MODELS', [RssSource])
    for model in all_models:
        for source in model.objects.all():
            collect_rss_items(None, source, check_user_rights=False)


def create_cms_article(user, item):
    """create a cms coop_cms.article from a RssItem"""
    article_class = get_article_class()
    content_type = ContentType.objects.get_for_model(article_class)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)

    if not (user.is_staff and user.has_perm(perm)):
        raise PermissionDenied

    art = article_class.objects.create(title=item.title, content=item.summary, publication=BaseArticle.DRAFT)
    item.processed = True
    item.save()
    return art



# -*- coding: utf-8 -*-

from django.conf.urls import re_path

from . import views

app_name = "newsletters"


urlpatterns = [
    re_path(
        r'^unregister/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.unregister_contact,
        name='unregister'
    ),
    re_path(
        r'^view-online/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.view_emailing_online,
        name='view_online'
    ),
    re_path(
        r'^view-online-lang/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/(?P<lang>\w+)/$',
        views.view_emailing_online_lang,
        name='view_online_lang'
    ),
    re_path(
        r'^link/(?P<link_uuid>[\w\d-]+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.view_link,
        name='view_link'
    ),
    re_path(
        r'^email-img/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.email_tracking,
        name='email_tracking'
    ),
]



# -*- coding: utf-8 -*-

from django.db import migrations, models

from coop_cms.moves import reverse


def set_homepage(apps, schema_editor):
    # Move from homepage_for_site to SiteSettings
    site_class = apps.get_model("sites", "Site")
    for site in site_class.objects.all():
        try:
            homepage_article = site.homepage_article.all()[0]
        except (models.ObjectDoesNotExist, IndexError, AttributeError):
            homepage_article = None

        if homepage_article:
            site_settings_class = apps.get_model("coop_cms", "SiteSettings")
            site_settings = site_settings_class.objects.get_or_create(site=site)[0]
            if not site_settings.homepage_url:
                homepage_url = reverse('coop_cms_view_article', args=[homepage_article.slug])
                site_settings.homepage_url = homepage_url
                site_settings.save()


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0003_auto_20160204_1540'),
    ]

    operations = [
        migrations.RunPython(set_homepage),
    ]



# -*- coding: utf-8 -*-
"""web utilities"""

import json
from urllib.parse import urlparse

from django.conf import settings
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseForbidden
from django.middleware.csrf import REASON_NO_REFERER, REASON_NO_CSRF_COOKIE
from django.shortcuts import render
from django.template.loader import get_template
from django.utils.translation import check_for_language, activate, get_language, ugettext as _
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView

from colorbox.decorators import popup_redirect

from coop_cms.forms.webutils import LanguageSelectionForm
from coop_cms.logger import logger
from coop_cms.moves import make_context
from coop_cms.settings import get_article_class, is_localized
from coop_cms.utils import strip_locale_path, make_locale_path


@csrf_exempt
def accept_cookies_message(request):
    """store if the user accept or not the cookies"""
    if request.method == 'POST':
        try:
            request.session['accept_cookies'] = int(request.POST.get('accept_cookies', 0)) == 1
        except (ValueError, TypeError):
            request.session['accept_cookies'] = False
        if request.session['accept_cookies']:
            message = _('Cookies and analytics are enabled')
        else:
            message = _('Cookies and analytics are disabled')
        data = {"ok": True, "message": message}
        return HttpResponse(json.dumps(data), content_type="application/json")
    raise Http404


def _do_change_language(request, lang_code, current_url):
    """change the language and returns redirect URL"""
    if lang_code and check_for_language(lang_code):

        # path is the locale-independent url
        path = strip_locale_path(current_url)[1]

        article_class = get_article_class()
        try:
            # get the translated slug of the current article
            # If switching from French to English and path is /fr/accueil/
            # The next should be : /en/home/

            # Get the article
            next_article = article_class.objects.get(slug=path.strip('/'))

        except article_class.DoesNotExist:
            next_article = None

        if hasattr(request, 'session'):
            request.session['django_language'] = lang_code
        activate(lang_code)

        if next_article:
            next_url = next_article.get_absolute_url()
        else:
            next_url = make_locale_path(path, lang_code)
        return next_url


@csrf_exempt
def change_language(request):
    """change the language"""

    if not is_localized():
        raise Http404

    next_url = request.POST.get('next', None) or request.GET.get('next', None)
    if not next_url:
        url = urlparse(request.META.get('HTTP_REFERER', ''))
        if url:
            next_url = url.path

    if request.method == 'POST':
        lang_code = request.POST.get('language', None)
        after_change_url = request.POST.get('next_url_after_change_lang', None)
        if after_change_url:
            next_url = after_change_url

        go_to_url = _do_change_language(request, lang_code, next_url)
        if go_to_url:
            next_url = go_to_url

    if not next_url:
        next_url = '/'

    return HttpResponseRedirect(next_url)


class DebugErrorCodeView(TemplateView):
    """Debugging: view error page in debug"""

    def get_template_names(self):
        """template to view"""
        return (
            "{0}.html".format(self.kwargs["error_code"]),
        )


def csrf_failure(request, reason=""):
    """
    Custom view used when request fails CSRF protection.

    ENABLED by default by coop_cms unless you set COOP_CMS_DO_NOT_INSTALL_CSRF_FAILURE_VIEW=True in settings.py
    """

    warn_text = "csrf_failure, reason: {0}".format(reason)
    logger.warn(warn_text)

    template = get_template('coop_cms/csrf_403.html')

    context = {
        'DEBUG': settings.DEBUG,
        'cookie_disabled': reason == REASON_NO_CSRF_COOKIE,
        'no_referer': reason == REASON_NO_REFERER,
    }

    html = template.render(make_context(request, context))

    return HttpResponseForbidden(html, content_type='text/html')


@popup_redirect
def switch_language_popup(request):
    """new article"""

    if request.method == "POST":
        form = LanguageSelectionForm(request.POST)
        if form.is_valid():
            lang_code = form.cleaned_data['language']
            url = urlparse(request.META.get('HTTP_REFERER', ''))
            if url:
                prev_url = url.path
            else:
                prev_url = ''
            url = _do_change_language(request, lang_code, prev_url)
            if url:
                return HttpResponseRedirect(url)

    else:
        form = LanguageSelectionForm(initial={'language': get_language()})

    return render(
        request,
        'coop_cms/popup_swicth_language.html',
        {'form': form}
    )



# -*- coding: utf-8 -*-

from django.conf import settings

from django.contrib.sites.models import Site
try:
    from django.urls import reverse
except:
    from django.core.urlresolvers import reverse
from django.test import RequestFactory
from django.test.utils import override_settings

from model_mommy import mommy

from colorbox.utils import assert_popup_redirects

from coop_cms.context_processors import homepage_url
from coop_cms.models import BaseArticle, SiteSettings
from coop_cms.settings import get_article_class
from coop_cms.shortcuts import get_headlines
from coop_cms.tests import BaseTestCase, UserBaseTestCase, BeautifulSoup


@override_settings(COOP_CMS_NO_HOMEPAGE=False)
class NoHomepageTest(UserBaseTestCase):
    
    @override_settings(COOP_CMS_NO_HOMEPAGE=True)
    def test_view_article_set_homepage_no_homepage(self):
        self._log_as_editor(can_add=True)
        article_class = get_article_class()
        art = mommy.make(article_class, slug="test", publication=BaseArticle.PUBLISHED)
        url = art.get_edit_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        url = reverse('coop_cms_set_homepage', args=[art.id])
        links = soup.select('.coop-bar a[href="{0}"]'.format(url))
        self.assertEqual(0, len(links))
        
    def test_view_article_set_homepage(self):
        self._log_as_editor(can_add=True)
        article_class = get_article_class()
        art = mommy.make(article_class, slug="test", publication=BaseArticle.PUBLISHED)
        url = art.get_edit_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        url = reverse('coop_cms_set_homepage', args=[art.id])
        links = soup.select('.coop-bar a[href="{0}"]'.format(url))
        self.assertEqual(1, len(links))


@override_settings(COOP_CMS_NO_HOMEPAGE=False, COOP_CMS_HOMEPAGE_NO_REDIRECTION=False)
class HomepageRedirectionTest(UserBaseTestCase):
    
    def setUp(self):
        super(HomepageRedirectionTest, self).setUp()
        self.site_id = settings.SITE_ID

    def tearDown(self):
        super(HomepageRedirectionTest, self).tearDown()
        settings.SITE_ID = self.site_id

    def test_homepage_url(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        article1_url = reverse('coop_cms_view_article', args=[article1.slug])
        mommy.make(SiteSettings, site=site, homepage_url=article1_url)
        self.assertEqual(article1.get_absolute_url(), article1_url)

    def test_homepage_url_context_processor(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        article2 = get_article_class().objects.create(title="django", content='django')
        article1_url = reverse('coop_cms_view_article', args=[article1.slug])
        mommy.make(SiteSettings, site=site, homepage_url=article1_url)
        request = RequestFactory().get(article2.get_absolute_url())
        self.assertEqual(homepage_url(request), {'homepage_url': article1.get_absolute_url()})
        
    def test_user_settings_homepage(self):
        site = Site.objects.get(id=settings.SITE_ID)
        a1 = get_article_class().objects.create(title="python", content='python')
        site_settings = mommy.make(SiteSettings, site=site, homepage_url=a1.get_absolute_url())
    
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].find(a1.get_absolute_url()) >= 0)
            
    def test_user_settings_homepage_priority(self):
        site = Site.objects.get(id=settings.SITE_ID)
        a1 = get_article_class().objects.create(title="python", content='python')
        a2 = get_article_class().objects.create(title="django", content='django', homepage_for_site=site)
        site_settings = mommy.make(SiteSettings, site=site, homepage_url=a1.get_absolute_url())

        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].find(a1.get_absolute_url()) >= 0)
    
    def test_user_settings_homepage_not_set(self):
        site = Site.objects.get(id=settings.SITE_ID)
        a1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        a2 = get_article_class().objects.create(title="django", content='django', homepage_for_site=site)
        site_settings = mommy.make(SiteSettings, site=site, homepage_url="")

        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].find(a2.get_absolute_url()) < 0)
        self.assertTrue(response['Location'].find(reverse('coop_cms_view_all_articles')) >= 0)

    def test_view_change_homepage(self):
        self._log_as_editor()
        a1 = get_article_class().objects.create(title="python", content='python')

        response = self.client.get(reverse('coop_cms_set_homepage', args=[a1.id]))
        self.assertEqual(response.status_code, 200)

        a1 = get_article_class().objects.get(id=a1.id)
        self.assertEqual(a1.is_homepage, False)
    
    def test_change_homepage(self):
        self._log_as_editor()
        site = Site.objects.get(id=settings.SITE_ID)
        a1 = get_article_class().objects.create(title="python", content='python')
        a2 = get_article_class().objects.create(title="django", content='django')
        a3 = get_article_class().objects.create(title="home1", content='homepage1')

        response = self.client.post(reverse('coop_cms_set_homepage', args=[a2.id]), data={'confirm': '1'})
        self.assertEqual(response.status_code, 200)
        a2 = get_article_class().objects.get(id=a2.id)
        home_url = reverse("coop_cms_homepage")

        assert_popup_redirects(response, home_url)
        self.assertEqual(a2.homepage_for_site, None)
        site_settings = SiteSettings.objects.get(site__id=settings.SITE_ID)
        self.assertEqual(site_settings.homepage_url, a2.get_absolute_url())
        self.assertEqual(site_settings.homepage_article, '')

        response = self.client.post(reverse('coop_cms_set_homepage', args=[a3.id]), data={'confirm': '1'})
        self.assertEqual(response.status_code, 200)
        home_url = reverse("coop_cms_homepage")
        assert_popup_redirects(response, home_url)

        a2 = get_article_class().objects.get(id=a2.id)
        a3 = get_article_class().objects.get(id=a3.id)
        site_settings = SiteSettings.objects.get(site__id=settings.SITE_ID)
        self.assertEqual(site_settings.homepage_url, a3.get_absolute_url())
        self.assertEqual(site_settings.homepage_article, '')

        self.assertEqual(a2.homepage_for_site, None)
        self.assertEqual(a3.homepage_for_site, None)
    
    def test_change_homepage_anonymous(self):
        Site.objects.get(id=settings.SITE_ID)
        a1 = get_article_class().objects.create(title="python", content='python')
        a2 = get_article_class().objects.create(title="django", content='django')
        a3 = get_article_class().objects.create(title="home1", content='homepage1')

        response = self.client.post(reverse('coop_cms_set_homepage', args=[a2.id]), data={'confirm': '1'}, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response.redirect_chain[0][0], 302)
        self.assertTrue(response.redirect_chain[-1][0].find(reverse('login')) >= 0)
        a2 = get_article_class().objects.get(id=a2.id)
        self.assertEqual(a2.homepage_for_site, None)
        self.assertEqual(0, SiteSettings.objects.count())

    def test_change_homepage_multisites(self):
        self._log_as_editor()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = Site.objects.create(domain="wooooooaa.com", name="wooaa")

        a1 = get_article_class().objects.create(title="python", content='python')
        a2 = get_article_class().objects.create(title="django", content='django')
        a3 = get_article_class().objects.create(title="home1", content='homepage1')

        settings.SITE_ID = site2.id
        a4 = get_article_class().objects.create(title="home1", content='homepage2')

        settings.SITE_ID = site1.id
        response = self.client.post(reverse('coop_cms_set_homepage', args=[a3.id]), data={'confirm': '1'})
        home_url = reverse("coop_cms_homepage")
        assert_popup_redirects(response, home_url)

        site_settings1 = SiteSettings.objects.get(site=site1)
        self.assertEqual(site_settings1.homepage_url, a3.get_absolute_url())

        settings.SITE_ID = site2.id
        response = self.client.post(reverse('coop_cms_set_homepage', args=[a4.id]), data={'confirm': '1'})
        self.assertEqual(response.status_code, 200)
        assert_popup_redirects(response, home_url)

        a4 = get_article_class().objects.get(id=a4.id)
        a3 = get_article_class().objects.get(id=a3.id)

        site_settings1 = SiteSettings.objects.get(site=site1)
        self.assertEqual(site_settings1.homepage_url, a3.get_absolute_url())

        site_settings2 = SiteSettings.objects.get(site=site2)
        self.assertEqual(site_settings2.homepage_url, a4.get_absolute_url())

    def test_homepage_multisites(self):
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = Site.objects.create(domain="wooooooaa.com", name="wooaa")

        article1 = get_article_class().objects.create(title="python", content='python')
        article2 = get_article_class().objects.create(title="django", content='django')
        article3 = get_article_class().objects.create(title="home1", content='homepage1')
        article4 = get_article_class().objects.create(title="home2", content='homepage2')

        self.assertEqual(0, SiteSettings.objects.count())

        SiteSettings.objects.create(site=site1, homepage_url=article3.get_absolute_url())

        SiteSettings.objects.create(site=site2, homepage_url=article4.get_absolute_url())

        settings.SITE_ID = site1.id
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].find(article3.get_absolute_url()) >= 0)
        self.assertEqual(article3.is_homepage, True)
        self.assertEqual(article4.is_homepage, False)

    def test_homepage_multisites_same_home(self):
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = Site.objects.create(domain="wooooooaa.com", name="wooaa")

        article1 = get_article_class().objects.create(title="python", content='python')
        article2 = get_article_class().objects.create(title="django", content='django')
        article3 = get_article_class().objects.create(title="home1", content='homepage1')
        article4 = get_article_class().objects.create(title="home2", content='homepage2')

        self.assertEqual(0, SiteSettings.objects.count())

        SiteSettings.objects.create(site=site1, homepage_url=article3.get_absolute_url())

        SiteSettings.objects.create(site=site2, homepage_url=article3.get_absolute_url())

        settings.SITE_ID = site1.id
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].find(article3.get_absolute_url()) >= 0)
        self.assertEqual(article3.is_homepage, True)
        self.assertEqual(article4.is_homepage, False)

        settings.SITE_ID = site2.id
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].find(article3.get_absolute_url()) >= 0)
        self.assertEqual(article3.is_homepage, True)
        self.assertEqual(article4.is_homepage, False)

    def test_edit_homepage(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        SiteSettings.objects.create(site=site, homepage_url=article.get_absolute_url())

        data = {"title": 'salut', 'content': 'bonjour!'}

        self._log_as_editor()

        response = self.client.get(article.get_edit_url())
        self.assertEqual(response.status_code, 200)

        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        article = get_article_class().objects.get(id=article.id)
        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.content, data['content'])

    def test_view_homepage(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        SiteSettings.objects.create(site=site, homepage_url=article.get_absolute_url())

        article_url = reverse('coop_cms_view_article', args=[article.slug])

        response = self.client.get(article_url)
        self.assertEqual(response.status_code, 200)

        response = self.client.get(reverse('coop_cms_homepage'), follow=True)
        self.assertRedirects(response, article.get_absolute_url())


@override_settings(COOP_CMS_NO_HOMEPAGE=False, COOP_CMS_HOMEPAGE_NO_REDIRECTION=False)
class HeadlineTest(BaseTestCase):

    def test_get_headlines_no_edit_perms(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.DRAFT, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=False)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT, headline=False)
        article5 = mommy.make(article_class, slug="test5", publication=BaseArticle.ARCHIVED, headline=True)
        article6 = mommy.make(article_class, slug="test6", publication=BaseArticle.ARCHIVED, headline=False)

        homepage = mommy.make(article_class, slug="test7", publication=BaseArticle.PUBLISHED, headline=False)
        site = Site.objects.get_current()
        SiteSettings.objects.create(site=site, homepage_url=homepage.get_absolute_url())

        headlines = list(get_headlines(homepage))
        self.assertEqual([article1], headlines)

    def test_get_headlines_edit_perms(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.DRAFT, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=False)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT, headline=False)
        article5 = mommy.make(article_class, slug="test5", publication=BaseArticle.ARCHIVED, headline=True)
        article6 = mommy.make(article_class, slug="test6", publication=BaseArticle.ARCHIVED, headline=False)

        homepage = mommy.make(article_class, slug="test7", publication=BaseArticle.PUBLISHED, headline=False)
        site = Site.objects.get_current()
        SiteSettings.objects.create(site=site, homepage_url=homepage.get_absolute_url())

        headlines = list(get_headlines(homepage, editable=True))
        self.assertEqual(sorted([article1, article2], key=lambda x: x.id), sorted(headlines, key=lambda x: x.id))

    def test_get_headlines_not_homepage(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.DRAFT, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=False)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT, headline=False)
        article5 = mommy.make(article_class, slug="test5", publication=BaseArticle.ARCHIVED, headline=True)
        article6 = mommy.make(article_class, slug="test6", publication=BaseArticle.ARCHIVED, headline=False)

        not_homepage = mommy.make(article_class, slug="test7", publication=BaseArticle.PUBLISHED, headline=False)

        headlines = list(get_headlines(not_homepage))
        self.assertEqual([], headlines)

    def test_get_headlines_other_homepage(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.DRAFT, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=False)
        article4 = mommy.make(article_class, slug="test4", publication=BaseArticle.DRAFT, headline=False)
        article5 = mommy.make(article_class, slug="test5", publication=BaseArticle.ARCHIVED, headline=True)
        article6 = mommy.make(article_class, slug="test6", publication=BaseArticle.ARCHIVED, headline=False)

        other_homepage = mommy.make(article_class, slug="test7", publication=BaseArticle.PUBLISHED, headline=False)

        site = mommy.make(Site)
        SiteSettings.objects.create(site=site, homepage_url=other_homepage.get_absolute_url())

        headlines = list(get_headlines(other_homepage))
        self.assertEqual([], headlines)

    def test_get_headlines_other_site(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=True)

        other_site = mommy.make(Site)
        article2.sites.clear()
        article2.sites.add(other_site)
        article2.save()

        article3.sites.clear()
        article3.save()

        homepage = mommy.make(article_class, slug="test4", publication=BaseArticle.PUBLISHED, headline=False)
        site = Site.objects.get_current()
        SiteSettings.objects.create(site=site, homepage_url=homepage.get_absolute_url())

        headlines = list(get_headlines(homepage))
        self.assertEqual([article1], headlines)

    @override_settings(COOP_CMS_HOMEPAGE_NO_REDIRECTION=True)
    def test_get_headlines_homepage_article(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=False)

        homepage = mommy.make(article_class, slug="test4", publication=BaseArticle.PUBLISHED, headline=False)
        site = Site.objects.get_current()
        SiteSettings.objects.create(site=site, homepage_article=homepage.slug)

        headlines = list(get_headlines(homepage))
        self.assertEqual(2, len(headlines))
        self.assertTrue(article1 in headlines)
        self.assertTrue(article2 in headlines)
        self.assertFalse(article3 in headlines)
        self.assertFalse(homepage in headlines)

    @override_settings(COOP_CMS_HOMEPAGE_NO_REDIRECTION=True)
    def test_get_headlines_homepage_article_other_sites(self):
        article_class = get_article_class()
        article1 = mommy.make(article_class, slug="test1", publication=BaseArticle.PUBLISHED, headline=True)
        article2 = mommy.make(article_class, slug="test2", publication=BaseArticle.PUBLISHED, headline=True)
        article3 = mommy.make(article_class, slug="test3", publication=BaseArticle.PUBLISHED, headline=True)

        other_site = mommy.make(Site)
        article2.sites.clear()
        article2.sites.add(other_site)
        article2.save()

        article3.sites.clear()
        article3.save()

        homepage = mommy.make(article_class, slug="test4", publication=BaseArticle.PUBLISHED, headline=False)
        site = Site.objects.get_current()
        SiteSettings.objects.create(site=site, homepage_article=homepage.slug)

        headlines = list(get_headlines(homepage))
        self.assertEqual([article1], headlines)


@override_settings(COOP_CMS_NO_HOMEPAGE=False, COOP_CMS_HOMEPAGE_NO_REDIRECTION=True)
class HomepageDirectTest(UserBaseTestCase):
    def setUp(self):
        super(HomepageDirectTest, self).setUp()
        self.site_id = settings.SITE_ID

    def tearDown(self):
        super(HomepageDirectTest, self).tearDown()
        settings.SITE_ID = self.site_id

    def test_homepage_url(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        article1_url = reverse('coop_cms_view_article', args=[article1.slug])
        mommy.make(SiteSettings, site=site, homepage_article=article1.slug)
        self.assertNotEqual(article1.get_absolute_url(), article1_url)
        self.assertEqual(article1.get_absolute_url(), reverse('coop_cms_homepage'))

    def test_homepage_url_context_processor(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        article2 = get_article_class().objects.create(title="django", content='django')
        mommy.make(SiteSettings, site=site, homepage_article=article1.slug)
        request = RequestFactory().get(article2.get_absolute_url())
        self.assertEqual(homepage_url(request), {'homepage_url': reverse('coop_cms_homepage')})

    def test_user_settings_homepage(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="python", content='python')
        mommy.make(SiteSettings, site=site, homepage_article=article1.slug)

        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article1.title)
        self.assertContains(response, article1.content)

    def test_user_settings_homepage_priority(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="python", content='python')
        get_article_class().objects.create(title="django", content='django', homepage_for_site=site)
        mommy.make(SiteSettings, site=site, homepage_article=article1.slug)

        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article1.title)
        self.assertContains(response, article1.content)

    def test_user_settings_both(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        article2 = get_article_class().objects.create(title="django", content='django')
        article2_url = reverse('coop_cms_view_article', args=[article2.slug])
        mommy.make(SiteSettings, site=site, homepage_url=article2_url, homepage_article=article1.slug)

        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article1.title)
        self.assertContains(response, article1.content)

    def test_user_settings_homepage_url(self):
        self._log_as_editor()
        site = Site.objects.get(id=settings.SITE_ID)
        article1 = get_article_class().objects.create(title="pythonx", content='pythonx')
        article2 = get_article_class().objects.create(title="django", content='django')
        article2_url = reverse('coop_cms_view_article', args=[article2.slug])
        mommy.make(SiteSettings, site=site, homepage_url=article2_url)

        response = self.client.get(reverse('coop_cms_homepage'), follow=True)
        self.assertRedirects(response, reverse('coop_cms_view_all_articles'))

    def test_view_change_homepage(self):
        self._log_as_editor()
        article1 = get_article_class().objects.create(title="python", content='python')

        response = self.client.get(reverse('coop_cms_set_homepage', args=[article1.id]))
        self.assertEqual(response.status_code, 200)

        article1 = get_article_class().objects.get(id=article1.id)
        self.assertEqual(article1.is_homepage, False)

    def test_change_homepage(self):
        self._log_as_editor()
        site = Site.objects.get(id=settings.SITE_ID)
        get_article_class().objects.create(title="python", content='python')
        article2 = get_article_class().objects.create(title="django", content='django')
        article3 = get_article_class().objects.create(title="home1", content='homepage1')

        response = self.client.post(reverse('coop_cms_set_homepage', args=[article2.id]), data={'confirm': '1'})
        self.assertEqual(response.status_code, 200)
        article2 = get_article_class().objects.get(id=article2.id)
        home_url = reverse("coop_cms_homepage")
        assert_popup_redirects(response, home_url)

        self.assertEqual(article2.homepage_for_site, None)
        site_settings = SiteSettings.objects.get(site__id=settings.SITE_ID)
        self.assertEqual(site_settings.homepage_article, article2.slug)
        self.assertEqual(site_settings.homepage_url, '')

        response = self.client.post(reverse('coop_cms_set_homepage', args=[article3.id]), data={'confirm': '1'})
        self.assertEqual(response.status_code, 200)
        home_url = reverse("coop_cms_homepage")
        assert_popup_redirects(response, home_url)

        article2 = get_article_class().objects.get(id=article2.id)
        article3 = get_article_class().objects.get(id=article3.id)
        site_settings = SiteSettings.objects.get(site__id=settings.SITE_ID)
        self.assertEqual(site_settings.homepage_article, article3.slug)
        self.assertEqual(site_settings.homepage_url, '')

        self.assertEqual(article2.homepage_for_site, None)
        self.assertEqual(article3.homepage_for_site, None)

    def test_change_homepage_anonymous(self):
        Site.objects.get(id=settings.SITE_ID)
        a1 = get_article_class().objects.create(title="python", content='python')
        a2 = get_article_class().objects.create(title="django", content='django')
        a3 = get_article_class().objects.create(title="home1", content='homepage1')

        response = self.client.post(reverse('coop_cms_set_homepage', args=[a2.id]), data={'confirm': '1'}, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response.redirect_chain[0][0], 302)
        self.assertTrue(response.redirect_chain[-1][0].find(reverse('login')) >= 0)
        a2 = get_article_class().objects.get(id=a2.id)
        self.assertEqual(a2.homepage_for_site, None)
        self.assertEqual(0, SiteSettings.objects.count())

    def test_change_homepage_multisites(self):
        self._log_as_editor()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = Site.objects.create(domain="wooooooaa.com", name="wooaa")

        a1 = get_article_class().objects.create(title="python", content='python')
        a2 = get_article_class().objects.create(title="django", content='django')
        a3 = get_article_class().objects.create(title="home1", content='homepage1')

        settings.SITE_ID = site2.id
        a4 = get_article_class().objects.create(title="home1", content='homepage2')

        settings.SITE_ID = site1.id
        response = self.client.post(reverse('coop_cms_set_homepage', args=[a3.id]), data={'confirm': '1'})
        home_url = reverse("coop_cms_homepage")
        assert_popup_redirects(response, home_url)

        site_settings1 = SiteSettings.objects.get(site=site1)
        self.assertEqual(site_settings1.homepage_url, '')
        self.assertEqual(site_settings1.homepage_article, a3.slug)

        settings.SITE_ID = site2.id
        response = self.client.post(reverse('coop_cms_set_homepage', args=[a4.id]), data={'confirm': '1'})
        self.assertEqual(response.status_code, 200)
        assert_popup_redirects(response, home_url)

        a4 = get_article_class().objects.get(id=a4.id)
        a3 = get_article_class().objects.get(id=a3.id)

        site_settings1 = SiteSettings.objects.get(site=site1)
        self.assertEqual(site_settings1.homepage_url, '')
        self.assertEqual(site_settings1.homepage_article, a3.slug)

        site_settings2 = SiteSettings.objects.get(site=site2)
        self.assertEqual(site_settings2.homepage_url, '')
        self.assertEqual(site_settings2.homepage_article, a4.slug)

    def test_homepage_multisites(self):
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = Site.objects.create(domain="wooooooaa.com", name="wooaa")

        article1 = get_article_class().objects.create(title="python", content='python')
        article2 = get_article_class().objects.create(title="django", content='django')
        article3 = get_article_class().objects.create(title="home1", content='homepage1')
        article4 = get_article_class().objects.create(title="home2", content='homepage2')

        self.assertEqual(0, SiteSettings.objects.count())

        SiteSettings.objects.create(site=site1, homepage_article=article3.slug)

        SiteSettings.objects.create(site=site2, homepage_article=article4.slug)

        settings.SITE_ID = site1.id
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article3.title)
        self.assertContains(response, article3.content)
        self.assertEqual(article3.is_homepage, True)
        self.assertEqual(article4.is_homepage, False)

    def test_homepage_multisites_same_home(self):
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = Site.objects.create(domain="wooooooaa.com", name="wooaa")

        article1 = get_article_class().objects.create(title="python", content='python')
        article2 = get_article_class().objects.create(title="django", content='django')
        article3 = get_article_class().objects.create(title="home1", content='homepage1')
        article4 = get_article_class().objects.create(title="home2", content='homepage2')

        self.assertEqual(0, SiteSettings.objects.count())

        SiteSettings.objects.create(site=site1, homepage_article=article3.slug)

        SiteSettings.objects.create(site=site2, homepage_article=article3.slug)
        article3.sites.add(site2)
        article3.save()

        settings.SITE_ID = site1.id
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article3.title)
        self.assertContains(response, article3.content)
        self.assertEqual(article3.is_homepage, True)
        self.assertEqual(article4.is_homepage, False)

        settings.SITE_ID = site2.id
        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article3.title)
        self.assertContains(response, article3.content)
        self.assertEqual(article3.is_homepage, True)
        self.assertEqual(article4.is_homepage, False)

    def test_edit_homepage(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        SiteSettings.objects.create(site=site, homepage_article=article.slug)

        data = {"title": 'salut', 'content': 'bonjour!'}

        self._log_as_editor()

        response = self.client.get(article.get_edit_url())
        self.assertEqual(response.status_code, 200)

        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        article = get_article_class().objects.get(id=article.id)
        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.content, data['content'])

    def test_view_homepage(self):
        site = Site.objects.get(id=settings.SITE_ID)
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        SiteSettings.objects.create(site=site, homepage_article=article.slug)

        article_url = reverse('coop_cms_view_article', args=[article.slug])

        response = self.client.get(article_url)
        self.assertEqual(response.status_code, 404)

        response = self.client.get(reverse('coop_cms_homepage'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)
        self.assertContains(response, article.content)



# -*- coding: utf-8 -*-
"""permission backend"""

class ArticlePermissionBackend(object):
    """Permission backend : check if a user is allowed to view/edit an editable object"""
    supports_object_permissions = True
    supports_anonymous_user = True
    supports_inactive_user = True

    def authenticate(self, *args, **kwargs):
        """authentication: return None --> no authentication through this backed"""
        return None

    def has_perm(self, user_obj, perm, obj=None):
        """has_perm : check if user is allowed to access obj for perm"""

        if obj:
            # get the 'perm' attribute of the object
            field = getattr(obj, perm, None)
            if field:
                if not callable(field):
                    # if an attribute : use the value
                    is_authorized = field
                else:
                    # if a method call it with user
                    is_authorized = field(user_obj)
                return is_authorized
        return False



# -*- coding: utf-8 -*-

from django.db import migrations, models


def set_links_sites(apps, schema_editor):
    # Associate every links with all sites
    site_class = apps.get_model("sites", "Site")
    link_class = apps.get_model("coop_cms", "Link")

    for link in link_class.objects.all():
        for site in site_class.objects.all():
            link.sites.add(site)
        link.save()


def reset_links_sites(apps, schema_editor):
    # Clear the sites for all links
    link_class = apps.get_model("coop_cms", "Link")

    for link in link_class.objects.all():
        link.sites.clear()
        link.save()


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0011_auto_20170502_1124'),
    ]

    operations = [
        migrations.RunPython(set_links_sites, reset_links_sites)
    ]




# -*- coding: utf-8 -*-

from django.conf import settings

try:
    from django.urls import reverse
except:
    from django.core.urlresolvers import reverse

from model_mommy import mommy

from coop_cms.models import BaseArticle
from coop_cms.settings import is_localized, get_article_class
from coop_cms.tests import BaseArticleTest, BeautifulSoup
from coop_cms.utils import get_model_app, get_model_name


class ArticleAdminTest(BaseArticleTest):
    
    def setUp(self):
        self.COOP_CMS_CAN_EDIT_ARTICLE_SLUG = getattr(settings, 'COOP_CMS_CAN_EDIT_ARTICLE_SLUG', None)
        
    def tearDown(self):
        setattr(settings, 'COOP_CMS_CAN_EDIT_ARTICLE_SLUG', self.COOP_CMS_CAN_EDIT_ARTICLE_SLUG)
    
    def test_slug_edition_draft(self):
        settings.COOP_CMS_CAN_EDIT_ARTICLE_SLUG = False
        
        self._log_as_staff_editor()
        
        article_class = get_article_class()
        
        article = mommy.make(article_class, slug="test", publication=BaseArticle.DRAFT)

        view_name = 'admin:%s_%s_change' % (get_model_app(article_class), get_model_name(article_class))
        url = reverse(view_name, args=[article.id])
        
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        if is_localized():
            from modeltranslation.utils import build_localized_fieldname
            for (lang, _name) in settings.LANGUAGES:
                field_name = build_localized_fieldname('slug', lang)
                self.assertEqual(soup.select("#id_" + field_name)[0]["type"], "text")
        else:
            self.assertEqual(soup.select("#id_slug")[0]["type"], "text")
                
    def test_slug_edition_published(self):
        settings.COOP_CMS_CAN_EDIT_ARTICLE_SLUG = False
        
        self._log_as_staff_editor()
        
        article_class = get_article_class()
        
        article = mommy.make(article_class, slug="test", publication=BaseArticle.PUBLISHED)
        
        view_name = 'admin:%s_%s_change' % (get_model_app(article_class), get_model_name(article_class))
        url = reverse(view_name, args=[article.id])
        
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        if is_localized():
            from modeltranslation.utils import build_localized_fieldname
            for (lang, _name) in settings.LANGUAGES:
                field_name = build_localized_fieldname('slug', lang)
                self.assertEqual(soup.select("#id_" + field_name)[0]["type"], "hidden")
        else:
            self.assertEqual(soup.select("#id_slug")[0]["type"], "hidden")
                
    def test_slug_edition_published_can_edit(self):
        settings.COOP_CMS_CAN_EDIT_ARTICLE_SLUG = True
        
        self._log_as_staff_editor()
        
        article_class = get_article_class()
        
        article = mommy.make(article_class, slug="test", publication=BaseArticle.PUBLISHED)
        
        view_name = 'admin:%s_%s_change' % (get_model_app(article_class), get_model_name(article_class))
        url = reverse(view_name, args=[article.id])
        
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        if is_localized():
            from modeltranslation.utils import build_localized_fieldname
            for (lang, _name) in settings.LANGUAGES:
                field_name = build_localized_fieldname('slug', lang)
                self.assertEqual(soup.select("#id_" + field_name)[0]["type"], "text")
        else:
            self.assertEqual(soup.select("#id_slug")[0]["type"], "text")



# -*- coding: utf-8 -*-

# from django.contrib.auth.models import User

# from coop_cms.tests import BaseTestCase as CoopCmsBaseTestCase
#
#
# class BaseTestCase(CoopCmsBaseTestCase):
#
#     def setUp(self):
#         super(BaseTestCase, self).setUp()
#         self.user = User.objects.create(username="toto")
#         self.user.set_password("abc")
#         self.user.is_staff = True
#         self.user.save()
#         self._login()
#
#     def _login(self):
#         return self.client.login(username="toto", password="abc")


# -*- coding: utf-8 -*-

from django.conf import settings

import logging
from bs4 import BeautifulSoup as BaseBeautifulSoup
import os.path
import shutil

from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse
from django.utils import timezone

from coop_cms.settings import get_article_class, get_unit_test_media_root, DEFAULT_MEDIA_ROOT


def make_dt(dt):
    if settings.USE_TZ:
        return timezone.make_aware(dt, timezone.get_default_timezone())
    else:
        return dt


#Used by a test below
def dummy_image_width(img):
    return 20


class BeautifulSoup(BaseBeautifulSoup):

    def __init__(self, content, parser='html.parser'):
        super(BeautifulSoup, self).__init__(content, parser)


@override_settings(MEDIA_ROOT=get_unit_test_media_root())
class BaseTestCase(TestCase):
    def _clean_files(self):
        if DEFAULT_MEDIA_ROOT != settings.MEDIA_ROOT:
            try:
                shutil.rmtree(settings.MEDIA_ROOT)
            except OSError:
                pass
        else:
            raise Exception("Warning! wrong media root for unit-testing")
    
    def setUp(self):
        logging.disable(logging.CRITICAL)
        self._clean_files()

    def tearDown(self):
        logging.disable(logging.NOTSET)
        self._clean_files()

    def assertNotAllowed(self, response):
        self.assertTrue(response.status_code in [302, 403])
        if response.status_code == 302:
            self.assertTrue(response['Location'].find(reverse('login')) >= 0)

    
class MediaBaseTestCase(BaseTestCase):

    def _get_file(self, file_name='unittest1.txt'):
        full_name = os.path.normpath(os.path.dirname(__file__) + '/fixtures/' + file_name)
        return open(full_name, 'rb')

    def get_safe_content(self, response):
        if hasattr(response, 'content'):
            if hasattr(response.content, 'read'):
                return response.content.read()
            else:
                return response.content
        elif hasattr(response, 'streaming_content'):
            return b"".join(response.streaming_content)
        return None

    def _log_as_mediamgr(self, is_staff=True, perm=None):
        u = User.objects.create(username='toto', is_staff=is_staff)
        u.set_password('toto')
        if perm:
            u.user_permissions.add(perm)
        u.save()
        logged = self.client.login(username='toto', password='toto')
        if not logged: raise Exception("Not logged")

    def _permission(self, code, model_class):
        ct = ContentType.objects.get_for_model(model_class)
        codename = '{0}_{1}'.format(code, ct.model)
        return Permission.objects.get(content_type__app_label=ct.app_label, codename=codename)


class UserBaseTestCase(BaseTestCase):

    def setUp(self):
        super(UserBaseTestCase, self).setUp()
        self.editor = None
        self.viewer = None

    def can_edit_newsletter_permission(self):
        return Permission.objects.get(content_type__app_label='coop_cms', codename='change_newsletter')

    def _log_as_editor(self, can_add=False):
        if not self.editor:
            self.editor = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
            self.editor.is_staff = True
            self.editor.is_active = True

            self.editor.user_permissions.add(self.can_edit_newsletter_permission())

            ct = ContentType.objects.get_for_model(get_article_class())
            codename = 'change_{0}'.format(ct.model)
            can_edit_article = Permission.objects.get(content_type__app_label=ct.app_label, codename=codename)
            self.editor.user_permissions.add(can_edit_article)

            if can_add:
                codename = 'add_{0}'.format(ct.model)
                can_add_article = Permission.objects.get(content_type__app_label=ct.app_label, codename=codename)
                self.editor.user_permissions.add(can_add_article)

            self.editor.save()

        return self.client.login(username='toto', password='toto')

    def _log_as_viewer(self):
        if not self.viewer:
            self.viewer = User.objects.create_user('titi', 'titi@toto.fr', 'titi')
            self.viewer.is_staff = True
            self.viewer.is_active = True
            self.viewer.user_permissions.add(self.can_edit_newsletter_permission())
            self.viewer.save()

        return self.client.login(username='titi', password='titi')


class BaseArticleTest(MediaBaseTestCase):
    def _log_as_editor(self):
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
        
        ct = ContentType.objects.get_for_model(get_article_class())
        
        perm = 'change_{0}'.format(ct.model)
        can_edit_article = Permission.objects.get(content_type=ct, codename=perm)
        user.user_permissions.add(can_edit_article)
        
        perm = 'add_{0}'.format(ct.model)
        can_add_article = Permission.objects.get(content_type=ct, codename=perm)
        user.user_permissions.add(can_add_article)
        
        user.is_active = True
        user.save()
        return self.client.login(username='toto', password='toto')
    
    def _log_as_staff_editor(self):
        self._log_as_editor()
        self.user.is_staff = True
        self.user.save()
    
    def _log_as_non_editor(self):
        self.regular_user = user = User.objects.create_user('zozo', 'zozo@toto.fr', 'zozo')
        
        user.is_active = True
        user.save()
        return self.client.login(username='zozo', password='zozo')
        
    def _log_as_editor_no_add(self):
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
        
        ct = ContentType.objects.get_for_model(get_article_class())
        
        perm = 'change_{0}'.format(ct.model)
        can_edit_article = Permission.objects.get(content_type=ct, codename=perm)
        user.user_permissions.add(can_edit_article)
        
        user.is_active = True
        user.save()
        
        return self.client.login(username='toto', password='toto')



# -*- coding: utf-8 -*-
"""forms"""

from django import forms
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.utils.timezone import now as dt_now
from django.utils.translation import ugettext as _, ugettext_lazy

import floppyforms.__future__ as floppyforms

from coop_cms.forms.base import InlineHtmlEditableModelForm
from coop_cms.bs_forms import Form as BsForm
from coop_cms.models import Newsletter, NewsletterSending, NewsletterItem
from coop_cms.settings import get_article_class, get_newsletter_templates
from coop_cms.widgets import ChosenSelectMultiple


class NewsletterItemAdminForm(forms.ModelForm):
    """admin form for NewsletterItem"""

    def __init__(self, *args, **kwargs):
        super(NewsletterItemAdminForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.item = kwargs.get('instance', None)
        article_choices = [(a.id, '{0}'.format(a)) for a in get_article_class().objects.all()]
        self.fields['object_id'] = forms.ChoiceField(
            choices=article_choices, required=True, help_text=_("Select an article")
        )
        self.fields['content_type'].required = False
        self.fields['content_type'].widget = forms.HiddenInput()

    def clean_content_type(self):
        """validation"""
        return ContentType.objects.get_for_model(get_article_class())


class NewsletterSettingsForm(forms.ModelForm):
    """Newsletter creation form"""

    class Meta:
        model = Newsletter
        fields = ('subject', 'template', 'newsletter_date', 'items', 'site')

    class Media:
        css = {
            'all': ('chosen/chosen.css', ),
        }
        js = (
            'chosen/chosen.jquery.js',
        )

    def __init__(self, user, *args, **kwargs):
        super(NewsletterSettingsForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        tpl_choices = get_newsletter_templates(None, user)
        if tpl_choices:
            self.fields["template"] = forms.ChoiceField(choices=tpl_choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["subject"].widget = forms.TextInput(attrs={'size': 30})
        self.fields["items"].widget.attrs["class"] = "chosen-select"
        choices = list(self.fields['items'].choices)
        sites_choices = []
        current_site = Site.objects.get_current()
        for choice in choices:
            obj_id = choice[0]
            obj = NewsletterItem.objects.get(id=obj_id)
            try:
                has_sites = getattr(obj.content_object, 'sites', None)
            except AttributeError:
                has_sites = False
            if has_sites:
                if current_site in obj.content_object.sites.all():
                    sites_choices.append(choice)
            else:
                sites_choices.append(choice)
        self.fields['items'].choices = sites_choices
        self.fields['items'].widget = ChosenSelectMultiple(
            choices=self.fields['items'].choices, force_template=True
        )

    def clean_items(self):
        """check items"""
        items = self.cleaned_data["items"]
        choice_ids = [choice[0] for choice in self.fields['items'].choices]
        for item in items:
            if item.id not in choice_ids:
                raise ValidationError(_("Invalid choice"))
        return items


class PublishArticleForm(forms.ModelForm):
    """Publish article form"""
    class Meta:
        model = get_article_class()
        fields = ('publication',)
        widgets = {
            'publication': forms.HiddenInput(),
        }


class NewsletterForm(InlineHtmlEditableModelForm):
    """form for newsletter edition"""

    class Meta:
        model = Newsletter
        fields = ('subject', 'content', )


class NewsletterSchedulingForm(floppyforms.ModelForm):
    """Newsletter scheduling"""
    class Meta:
        model = NewsletterSending
        fields = ('scheduling_dt',)

    def clean_scheduling_dt(self):
        """validation"""
        sch_dt = self.cleaned_data['scheduling_dt']

        if not sch_dt:
            raise ValidationError(_("This field is required"))

        if sch_dt < dt_now():
            raise ValidationError(_("The scheduling date must be in future"))

        return sch_dt


class NewsletterTemplateForm(forms.Form):
    """Newsletter template"""

    def __init__(self, newsletter, user, *args, **kwargs):
        super(NewsletterTemplateForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        choices = get_newsletter_templates(newsletter, user)
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["template"].initial = newsletter.template
        

class NewsletterAdminForm(forms.ModelForm):
    """newsletter admin form"""
    def __init__(self, *args, **kwargs):
        super(NewsletterAdminForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.newsletter = kwargs.get('instance', None)
        choices = get_newsletter_templates(self.newsletter, getattr(self, "current_user", None))
        if choices:
            self.fields["template"] = forms.ChoiceField(choices=choices)
        else:
            self.fields["template"] = forms.CharField()
        self.fields["items"].widget.attrs["class"] = "chosen-select"

    class Meta:
        model = Newsletter
        fields = ('subject', 'content', 'template', 'source_url', 'items', 'newsletter_date', 'site')
        widgets = {}

    class Media:
        css = {
            'all': ('css/admin-tricks.css', 'chosen/chosen.css', ),
        }
        js = (
            'chosen/chosen.jquery.js',
        )

        
class NewsletterHandleRecipients(BsForm):
    email_help_text = ugettext_lazy("Enter another address to send to someone who is not in the list")
    email_label = ugettext_lazy("Email")

    emails = forms.MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple(), required=False,
        label=_(ugettext_lazy('Emails')), help_text=ugettext_lazy('Check the address if you want to send it the test'),
        choices=[]
    )
    additional_email1 = forms.EmailField(required=False, label=email_label, help_text=email_help_text)
    additional_email2 = forms.EmailField(required=False, label=email_label, help_text=email_help_text)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices = []
        for elt in getattr(settings, 'COOP_CMS_TEST_EMAILS', []):
            if isinstance(elt, str):
                choices.append((elt, elt))
            else:
                # tuple or list
                choices.append(elt)
        self.fields['emails'].choices = choices

    def clean(self):
        super().clean()
        emails = self.cleaned_data['emails']
        additional_email1 = self.cleaned_data['additional_email1']
        additional_email2 = self.cleaned_data['additional_email2']
        if not emails and not additional_email1 and not additional_email2:
            raise forms.ValidationError(_('Please select or enter at least one email'))



# -*- coding: utf-8 -*-
"""test email sending"""

from datetime import datetime
from unittest import skipIf

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import management
from django.core import mail
from django.test.utils import override_settings
from django.urls import reverse
from django.utils.translation import activate

from coop_cms.models import Newsletter
from coop_cms.tests import BaseTestCase
from coop_cms.utils import get_url_in_language
from model_mommy import mommy

from .. import models


class SendEmailingTest(BaseTestCase):

    def setUp(self):
        activate(settings.LANGUAGES[0][0])
        super(SendEmailingTest, self).setUp()
        settings.COOP_CMS_FROM_EMAIL = 'toto@toto.fr'
        settings.COOP_CMS_REPLY_TO = 'titi@toto.fr'
        site = Site.objects.get_current()
        site.domain = settings.COOP_CMS_SITE_PREFIX
        site.save()

    def tearDown(self):
        activate(settings.LANGUAGES[0][0])

    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter(self):
        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, email=name + '@toto.fr')
            for name in names
        ]
        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'
        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()
        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site)
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()
        management.call_command('send_newsletters', verbosity=0)
        emailing = models.Emailing.objects.get(id=emailing.id)
        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))
        self.assertEqual(len(mail.outbox), len(contacts))
        outbox = list(mail.outbox)
        outbox.sort(key=lambda _elt: _elt.to)
        contacts.sort(key=lambda _contact: _contact.email)
        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:unregister', args=[emailing.id, contact.uuid]
            )
            self.assertEqual(email.to, [contact.email])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
            # Check magic links
            self.assertTrue(models.MagicLink.objects.count() > 0)

    @skipIf(len(settings.LANGUAGES) < 2, "LANGUAGES less than 2")
    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_language(self):
        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(
                models.Contact,
                email=name+'@toto.fr',
                last_name=name.capitalize(),
                first_name=name,
            ) for name in names
        ]
        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        content = '<h2>Hello #!-fullname-!#!</h2><p>{0}Visit <a href="http://toto.{0}">{0}</a>'
        content += '<a href="mailto:me@me.{0}">mailme</a><a href="#art1">internal link</a></p>'
        newsletter_data = {
            'subject_' + origin_lang: 'This is the {0} subject'.format(origin_lang),
            'subject_' + trans_lang: 'This is the {0} subject'.format(trans_lang),
            'content_' + origin_lang: content.format(origin_lang, ),
            'content_' + trans_lang: content.format(trans_lang),
            'template': 'test/newsletter_contact_lang.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()
        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site, lang=trans_lang),
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()
        management.call_command('send_newsletters', verbosity=0)
        emailing = models.Emailing.objects.get(id=emailing.id)
        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))
        self.assertEqual(len(mail.outbox), len(contacts))
        outbox = list(mail.outbox)
        outbox.sort(key=lambda _elt: _elt.to)
        contacts.sort(key=lambda _contact: _contact.email)
        activate(trans_lang)
        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:unregister', args=[emailing.id, contact.uuid]
            )
            view_en_url = reverse("newsletters:view_online_lang", args=[emailing.id, contact.uuid, 'en'])
            self.assertEqual(email.to, [contact.email])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject_' + trans_lang])
            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            # Check links are not magic
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(view_en_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.{0}".format(trans_lang)) > 0)
            # Check internal links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
            # Check magic links
            self.assertTrue(models.MagicLink.objects.count() > 0)

    @skipIf(len(settings.LANGUAGES) < 2, "LANGUAGES less than 2")
    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_contact_language(self):
        """test that we use the favorite language of the contact when sending him a newsletter"""
        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        names = ['alpha', 'beta', 'gamma']
        langs = ['', origin_lang, trans_lang]
        contacts = [
            mommy.make(
                models.Contact,
                email=name+'@toto.fr',
                last_name=name.capitalize(),
                first_name=name,
                favorite_language=lang
            ) for (name, lang) in zip(names, langs)
        ]
        content = '<h2>Hello #!-fullname-!#!</h2><p>{0}Visit <a href="http://toto.{0}">{0}</a>'
        content += '<a href="mailto:me@me.{0}">mailme</a><a href="#art1">internal link</a></p>'
        newsletter_data = {
            'subject_' + origin_lang: 'This is the {0} subject'.format(origin_lang),
            'subject_' + trans_lang: 'This is the {0} subject'.format(trans_lang),
            'content_' + origin_lang: content.format(origin_lang),
            'content_' + trans_lang: content.format(trans_lang),
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()
        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site, lang=''),
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()
        management.call_command('send_newsletters', verbosity=0)
        emailing = models.Emailing.objects.get(id=emailing.id)
        activate(origin_lang)
        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))
        self.assertEqual(len(mail.outbox), len(contacts))
        outbox = list(mail.outbox)
        outbox.sort(key=lambda _elt: _elt.to)
        contacts.sort(key=lambda _contact: _contact.email)
        activate(trans_lang)
        for email, contact, lang in zip(outbox, contacts, langs):
            viewonline_url = reverse(
                'newsletters:view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = reverse(
                'newsletters:unregister', args=[emailing.id, contact.uuid]
            )
            contact_lang = lang or origin_lang
            viewonline_url = get_url_in_language(viewonline_url, contact_lang)
            unsubscribe_url = get_url_in_language(unsubscribe_url, contact_lang)
            viewonline_url = emailing.get_domain_url_prefix() + viewonline_url
            unsubscribe_url = emailing.get_domain_url_prefix() + unsubscribe_url
            self.assertEqual(email.to, [contact.email])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject_' + contact_lang])
            self.assertEqual(email.extra_headers.get('Reply-To', ''), '')
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, email.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.{0}".format(contact_lang)) > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
            # Check magic links
            self.assertTrue(models.MagicLink.objects.count() > 0)

    @override_settings(COOP_CMS_REPLY_TO="reply_to@toto.fr")
    def test_send_newsletter_reply_to(self):
        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, email=name+'@toto.fr', last_name=name.capitalize())
            for name in names
        ]
        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'
        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()
        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site)
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()
        management.call_command('send_newsletters', verbosity=0)
        emailing = models.Emailing.objects.get(id=emailing.id)
        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))
        self.assertEqual(len(mail.outbox), len(contacts))
        outbox = list(mail.outbox)
        outbox.sort(key=lambda _elt: _elt.to)
        contacts.sort(key=lambda _contact: _contact.email)
        for email, contact in zip(outbox, contacts):
            viewonline_url = reverse(
                'newsletters:view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = reverse(
                'newsletters:unregister', args=[emailing.id, contact.uuid]
            )
            viewonline_url = emailing.get_domain_url_prefix() + viewonline_url
            unsubscribe_url = emailing.get_domain_url_prefix() + unsubscribe_url

            self.assertEqual(email.to, [contact.email])
            self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertEqual(email.extra_headers['Reply-To'], settings.COOP_CMS_REPLY_TO)
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, settings.COOP_CMS_REPLY_TO)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
            # Check magic links
            self.assertTrue(models.MagicLink.objects.count() > 0)

    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_from_email(self):
        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, email=name+'@toto.fr', last_name=name.capitalize())
            for name in names
        ]
        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'
        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()
        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site, from_email="abcd@defg.fr"),
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()
        management.call_command('send_newsletters', verbosity=0)
        emailing = models.Emailing.objects.get(id=emailing.id)
        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))
        self.assertEqual(len(mail.outbox), len(contacts))
        outbox = list(mail.outbox)
        outbox.sort(key=lambda _elt: _elt.to)
        contacts.sort(key=lambda _contact: _contact.email)
        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:unregister', args=[emailing.id, contact.uuid]
            )
            self.assertEqual(email.to, [contact.email])
            self.assertEqual(email.from_email, emailing.from_email)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, emailing.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
            # Check magic links
            self.assertTrue(models.MagicLink.objects.count() > 0)

    @override_settings(COOP_CMS_REPLY_TO="")
    def test_send_newsletter_check_unregister_name(self):
        names = ['alpha', 'beta', 'gamma']
        contacts = [
            mommy.make(models.Contact, email=name + '@toto.fr', last_name=name.capitalize())
            for name in names
        ]
        content = '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a>'
        content += '<a href="mailto:me@me.fr">mailme</a><a href="#art1">internal link</a></p>'
        newsletter_data = {
            'subject': 'This is the subject',
            'content': content,
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()
        subscription_1 = mommy.make(models.SubscriptionType, site=site, name="MY_COMPANY_#1")
        subscription_2 = mommy.make(models.SubscriptionType, site=site, name="MY_COMPANY_#2", from_email="abcd@defg.fr")
        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=subscription_2,
        )
        for contact in contacts:
            emailing.send_to.add(contact)
        emailing.save()
        management.call_command('send_newsletters', verbosity=0)
        emailing = models.Emailing.objects.get(id=emailing.id)
        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), len(contacts))
        self.assertEqual(len(mail.outbox), len(contacts))
        outbox = list(mail.outbox)
        outbox.sort(key=lambda _elt: _elt.to)
        contacts.sort(key=lambda _contact: _contact.email)
        for email, contact in zip(outbox, contacts):
            viewonline_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:view_online', args=[emailing.id, contact.uuid]
            )
            unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                'newsletters:unregister', args=[emailing.id, contact.uuid]
            )
            self.assertFalse(email.body.find(subscription_1.name) >= 0)
            self.assertTrue(email.body.find(subscription_2.name) >= 0)
            self.assertEqual(email.to, [contact.email])
            self.assertEqual(email.from_email, emailing.from_email)
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertEqual(
                email.extra_headers['List-Unsubscribe'],
                '<{0}>, <mailto:{1}?subject=unsubscribe>'.format(unsubscribe_url, emailing.from_email)
            )
            self.assertTrue(email.body.find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find(contact.fullname) >= 0)
            self.assertTrue(email.alternatives[0][0].find(viewonline_url) >= 0)
            self.assertTrue(email.alternatives[0][0].find(unsubscribe_url) >= 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("mailto:me@me.fr") > 0)
            # Check mailto links are not magic
            self.assertTrue(email.alternatives[0][0].find("#art1") > 0)
            # Check magic links
            self.assertTrue(models.MagicLink.objects.count() > 0)



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        ('coop_cms', '0010_auto_20170320_1349'),
    ]

    operations = [
        migrations.AddField(
            model_name='link',
            name='sites',
            field=models.ManyToManyField(to='sites.Site', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""forms"""

from django.utils.translation import ugettext_lazy as _

from coop_cms.bs_forms import Form as BsForm
import floppyforms.__future__ as forms


class UnregisterForm(BsForm):
    """User wants to unregister from emailing"""
    reason = forms.CharField(required=False, widget=forms.Textarea, label=_("Reason"))



# -*- coding: utf-8 -*-

from django.conf import settings

from coop_cms.moves import SkipTest

if 'coop_cms.apps.test_app' in settings.INSTALLED_APPS:
    from coop_cms.apps.test_app.tests import GenericViewTestCase as BaseGenericViewTestCase
else:
    from coop_cms.tests import BaseTestCase as BaseGenericViewTestCase


class GenericViewTestCase(BaseGenericViewTestCase):
    warning = """
    Add this to your settings.py to enable this test:
    if len(sys.argv)>1 and 'test' == sys.argv[1]:
        INSTALLED_APPS = INSTALLED_APPS + ('coop_cms.apps.test_app',)
    """
    
    def setUp(self):
        super(GenericViewTestCase, self).setUp()
        if not ('coop_cms.apps.test_app' in settings.INSTALLED_APPS):
            raise SkipTest('coop_cms.apps.test_app not installed')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-09 21:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0015_auto_20170608_1600'),
    ]

    operations = [
        migrations.AlterField(
            model_name='articlecategory',
            name='sites',
            field=models.ManyToManyField(default=[], to='sites.Site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='newsletter',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.Site', verbose_name='site'),
        ),
    ]



# -*- coding: utf-8 -*-
"""utils"""

from bs4 import BeautifulSoup

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import get_connection, EmailMultiAlternatives
from django.template.loader import get_template
from django.utils import translation

from coop_cms.settings import get_newsletter_context_callbacks, get_eastern_languages

from .text import dehtml
from .i18n import activate_lang, get_language


def strip_a_tags(pretty_html_text):
    """
    Reformat prettified html to remove spaces in <a> tags
    """
    pos = 0
    fixed_html = ''
    while True:
        new_pos = pretty_html_text.find('<a', pos)
        if new_pos > 0:
            fixed_html += pretty_html_text[pos:new_pos]
            end_tag = pretty_html_text.find('>', new_pos + 1)
            end_pos = pretty_html_text.find('</a>', end_tag + 1)

            fixed_html += pretty_html_text[new_pos:end_tag + 1]
            tag_content = pretty_html_text[end_tag + 1:end_pos]
            fixed_html += tag_content.strip() + '</a>'

            pos = end_pos + 4
        else:
            fixed_html += pretty_html_text[pos:]
            break

    return fixed_html


def _replace_from_end(s, a, b, times=None):
    """replace from end"""
    return s[::-1].replace(a, b, times)[::-1]


def avoid_line_too_long(pretty_html_text):
    """
    detect any line with more than 998 characters
    """
    lines = pretty_html_text.split('\n')
    new_lines = []
    for line in lines:
        line_length = len(line)
        if line_length >= 998:
            # Cut the line in several parts of 900 characters
            parts = []
            part_size = 900
            part_index = 0
            while part_size * len(parts) < line_length:
                parts.append(line[part_index*part_size:(part_index + 1)*part_size])
                part_index = len(parts)
            parts = [_replace_from_end(part, ' ', '\n', 1) for part in parts]
            new_lines.append(''.join(parts))
        else:
            new_lines.append(line)
    return '\n'.join(new_lines)


def make_links_absolute(html_content, newsletter=None, site_prefix=""):
    """replace all local url with site_prefixed url"""
    
    def make_abs(url):
        """make absolute url"""
        if url.startswith('..'):
            url = url[2:]
        while url.startswith('/..'):
            url = url[3:]
        if url.startswith('/'):
            url = '%s%s' % (site_prefix, url)
        return url

    if not site_prefix:
        if newsletter:
            site_prefix = newsletter.get_site_prefix()
        else:
            site = Site.objects.get_current()
            site_prefix = "http://{0}".format(site.domain)

    soup = BeautifulSoup(html_content, 'html.parser')
    for a_tag in soup.find_all("a"):
        if a_tag.get("href", None):
            a_tag["href"] = make_abs(a_tag["href"])
    
    for img_tag in soup.find_all("img"):
        if img_tag.get("src", None):
            img_tag["src"] = make_abs(img_tag["src"])

    pretty_html = soup.prettify()
    fixed_html = strip_a_tags(pretty_html)
    return avoid_line_too_long(fixed_html)


def _send_email(subject, html_text, dests, list_unsubscribe):
    """send an email"""
    emails = []
    connection = get_connection()
    from_email = getattr(settings, 'COOP_CMS_FROM_EMAIL', settings.DEFAULT_FROM_EMAIL)
    reply_to = getattr(settings, 'COOP_CMS_REPLY_TO', None)

    # make header
    headers = {}
    if reply_to:
        headers['Reply-To'] = reply_to
    if list_unsubscribe:
        headers['List-Unsubscribe'] = ", ".join(["<{0}>".format(url) for url in list_unsubscribe])

    for address in dests:
        text = dehtml(html_text)
        email = EmailMultiAlternatives(subject, text, from_email, [address], headers=headers)
        email.attach_alternative(html_text, "text/html")
        emails.append(email)
    return connection.send_messages(emails)


def send_email(subject, template_name, context, site_prefix, dests, lang=None, list_unsubscribe=None):
    """Send an HTML email"""
    activate_lang(lang)

    the_template = get_template(template_name)

    try:
        html_text = the_template.render(context)
    except Exception:
        # import traceback
        # print traceback.print_exc()
        raise

    html_text = make_links_absolute(html_text, site_prefix=site_prefix)

    return _send_email(subject, html_text, dests, list_unsubscribe=list_unsubscribe)


def send_newsletter(newsletter, dests, list_unsubscribe=None):
    """
    send newsletter
    newsletter : a newsletter object
    dests : the list of recipients
    list_unsubscribe : a list of url for unsubscribe
    """

    # Force the newsletter as public
    newsletter.is_public = True
    newsletter.save()

    lang = get_language()
    if not (lang in [code_and_name[0] for code_and_name in settings.LANGUAGES]):
        # The current language is not defined in settings.LANGUAGE
        # force it to the defined language
        lang = settings.LANGUAGE_CODE[:2]
        translation.activate(lang)

    the_template = get_template(newsletter.get_template_name())
    context_dict = {
        'title': newsletter.subject,
        'newsletter': newsletter,
        'by_email': True,
        'SITE_PREFIX': settings.COOP_CMS_SITE_PREFIX,
        'MEDIA_URL': settings.MEDIA_URL,
        'STATIC_URL': settings.STATIC_URL,
    }

    for callback in get_newsletter_context_callbacks():
        data = callback(newsletter)
        if data:
            context_dict.update(data)

    try:
        html_text = the_template.render(context_dict)
    except Exception:
        # import traceback
        # print(traceback.print_exc())
        raise

    html_text = make_links_absolute(html_text, newsletter)
    subject = ' '.join(newsletter.subject.split('\n'))
    subject = ' '.join(subject.split('\r'))

    return _send_email(subject, html_text, dests, list_unsubscribe)



# -*- coding: utf-8 -*-
"""
Email authentication
"""

from django.apps import AppConfig


class EmailAuthAppConfig(AppConfig):
    name = 'coop_cms.apps.email_auth'
    verbose_name = "coop CMS > Email authentication"



# -*- coding: utf-8 -*-
"""navigation tags"""

from django import template
from django.contrib.contenttypes.models import ContentType
from django.template import VariableDoesNotExist
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from coop_cms.models import NavNode
from coop_cms.settings import get_navtree_class

register = template.Library()


def extract_kwargs(args):
    """args from templatetag args"""
    kwargs = {}
    for arg in args:
        try:
            key, value = arg.split('=')
            kwargs[key] = value
        except ValueError:  # No = in the arg
            pass
    return kwargs


class NavigationTemplateNode(template.Node):
    """Navigation templatetag"""
    def __init__(self, *args, **kwargs):
        super(NavigationTemplateNode, self).__init__()
        self._kwargs = {}
        for (key, value) in kwargs.items():
            self._kwargs[key] = template.Variable(value)

    def resolve_kwargs(self, context):
        """resolve: get value from context or from string"""
        kwargs = {}
        for (key, value) in self._kwargs.items():
            try:
                kwargs[key] = value.resolve(context)
            except VariableDoesNotExist:
                kwargs[key] = value.var  # if the variable can not be resolved, take the value as is

        if not 'tree' in kwargs:
            kwargs['tree'] = 'default'

        tree = get_navtree_class().objects.get_or_create(name=kwargs['tree'])[0]
        if 'coop_cms_navtrees' in context.dicts[0]:
            context.dicts[0]['coop_cms_navtrees'].append(tree)
        else:
            context.dicts[0]['coop_cms_navtrees'] = [tree]

        return kwargs


class NavigationAsNestedUlNode(NavigationTemplateNode):
    """Navigation as nested ul"""

    def __init__(self, **kwargs):
        super(NavigationAsNestedUlNode, self).__init__(**kwargs)

    def render(self, context):
        """to html"""
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        root_nodes = NavNode.objects.filter(tree__name=tree_name)
        parent = kwargs.pop('parent', None)
        if parent:
            root_nodes = root_nodes.filter(parent=parent)
        else:
            root_nodes = root_nodes.filter(parent__isnull=True)
        root_nodes = root_nodes.order_by("ordering")
        total_nodes = root_nodes.count()    
        return ''.join([
            node.as_navigation(node_pos=i + 1, total_nodes=total_nodes, **kwargs)
            for (i, node) in enumerate(root_nodes)
        ])


@register.tag
def navigation_as_nested_ul(parser, token):
    """template tag"""
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    return NavigationAsNestedUlNode(**kwargs)


class NavigationBreadcrumbNode(NavigationTemplateNode):
    """Navigation"""
    def __init__(self, obj, **kwargs):
        super(NavigationBreadcrumbNode, self).__init__(**kwargs)
        self.object_var = template.Variable(obj)

    def render(self, context):
        """to html"""
        obj = self.object_var.resolve(context)
        content_type = ContentType.objects.get_for_model(obj.__class__)
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        nav_nodes = NavNode.objects.filter(tree__name=tree_name, content_type=content_type, object_id=obj.id)
        if nav_nodes.count() > 0:
            return nav_nodes[0].as_breadcrumb(**kwargs)
        return ''


@register.tag
def navigation_breadcrumb(parser, token):
    """Navigation"""
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    if len(args) < 2:
        raise template.TemplateSyntaxError(_("navigation_breadcrumb requires object as argument"))
    return NavigationBreadcrumbNode(args[1], **kwargs)


class NavigationChildrenNode(NavigationTemplateNode):
    """Navigation"""

    def __init__(self, obj, **kwargs):
        super(NavigationChildrenNode, self).__init__(**kwargs)
        self.object_var = template.Variable(obj)

    def render(self, context):
        """to html"""
        obj = self.object_var.resolve(context)
        content_type = ContentType.objects.get_for_model(obj.__class__)
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        nav_nodes = NavNode.objects.filter(tree__name=tree_name, content_type=content_type, object_id=obj.id)
        if nav_nodes.exists():
            return nav_nodes[0].children_as_navigation(**kwargs)
        return ''


@register.tag
def navigation_children(parser, token):
    """Navigation"""
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    if len(args) < 2:
        raise template.TemplateSyntaxError(
            _("navigation_children requires object as argument and optionally tree={{tree_name}}")
        )
    return NavigationChildrenNode(args[1], **kwargs)


class NavigationSiblingsNode(NavigationTemplateNode):
    """Navigation"""

    def __init__(self, obj, **kwargs):
        super(NavigationSiblingsNode, self).__init__(**kwargs)
        self.object_var = template.Variable(obj)

    def render(self, context):
        obj = self.object_var.resolve(context)
        content_type = ContentType.objects.get_for_model(obj.__class__)
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        nav_nodes = NavNode.objects.filter(tree__name=tree_name, content_type=content_type, object_id=obj.id)
        if nav_nodes.count() > 0:
            return nav_nodes[0].siblings_as_navigation(**kwargs)
        return ''


@register.tag
def navigation_siblings(parser, token):
    """Navigation"""
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    if len(args) < 2:
        raise template.TemplateSyntaxError(_("navigation_siblings requires object as argument"))
    return NavigationSiblingsNode(args[1], **kwargs)


DEFAULT_NAVROOT_TEMPLATE = 'coop_cms/navigation_node.html'


@register.filter
def render_template_node(node, template_name=""):
    """render to html"""
    the_template = get_template(template_name or DEFAULT_NAVROOT_TEMPLATE)
    return the_template.render({'node': node})


class NavigationRootNode(NavigationTemplateNode):
    """Navigation"""

    def render(self, context):
        """to html"""
        kwargs = self.resolve_kwargs(context)
        tree_name = kwargs.pop('tree', 'default')
        template_name = kwargs.pop('template_name', DEFAULT_NAVROOT_TEMPLATE)
        parent = kwargs.pop('parent', None)

        root_nodes = NavNode.objects.filter(tree__name=tree_name, in_navigation=True)
        if parent:
            root_nodes = root_nodes.filter(parent=parent)
        else:
            root_nodes = root_nodes.filter(parent__isnull=True)
        root_nodes = root_nodes.order_by("ordering")

        return ''.join([render_template_node(node, template_name) for node in root_nodes])


@register.tag
def navigation_root_nodes(parser, token):
    """navigation"""
    args = token.contents.split()
    kwargs = extract_kwargs(args)
    return NavigationRootNode(**kwargs)


@register.filter
def get_navigation_node(nav_node_id):
    """navigation"""
    return NavNode.objects.get(id=nav_node_id)



# -*- coding: utf-8 -*-

from django.db import migrations, models
import datetime
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0002_auto_20160108_1628'),
    ]

    operations = [
        migrations.AddField(
            model_name='newsletter',
            name='created',
            field=django_extensions.db.fields.CreationDateTimeField(default=datetime.datetime(2016, 2, 4, 15, 40, 3, 30106), verbose_name='created', auto_now_add=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='newsletter',
            name='modified',
            field=django_extensions.db.fields.ModificationDateTimeField(default=datetime.datetime(2016, 2, 4, 15, 40, 22, 708986), verbose_name='modified', auto_now=True),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-

from coop_cms.tests import BaseArticleTest
from coop_cms.utils import dehtml


class DehtmlTest(BaseArticleTest):
    
    def test_simple_text(self):
        text = "This is a simple text"
        self.assertEqual(dehtml(text), text)

    def test_paragraph(self):
        text = "This is a simple text"
        html_text = '<p>{0}</p>'.format(text)
        self.assertEqual(dehtml(html_text), text)

    def test_paragraph_inside(self):
        text = "<h1>This is a title</h1><p>This is a paragraph</p><p>This is another paragraph</p>"
        self.assertEqual(dehtml(text), "This is a title\n\nThis is a paragraph\n\nThis is another paragraph")

    def test_nbsp(self):
        text = "This is a simple text"
        html_text = text.replace(' ', '&nbsp;')
        self.assertNotEqual(html_text, text)
        self.assertEqual(dehtml(html_text), text)

    def test_gt_lt(self):
        text = "This is a simple text"
        html_text = '&gt;' + text + "&lt;"
        self.assertEqual(dehtml(html_text), ">" + text + "<")

    def test_special_chars(self):
        text = "This &ouml;is &auml; simple text&uuml;"
        html_text = "This &ouml;is &auml; simple text&uuml;"
        self.assertEqual(dehtml(html_text, allow_html_chars=True), text)

    def test_charset_chars(self):
        text = "à l'Opéra Grand Avignon"
        html_text = "<p>&agrave; l&#39;Op&eacute;ra Grand Avignon</p>"
        self.assertEqual(dehtml(html_text, allow_html_chars=False), text)

    def test_charset_chars_allowed(self):
        text = "&agrave; l&#39;Op&eacute;ra Grand Avignon"
        html_text = "<p>&agrave; l&#39;Op&eacute;ra Grand Avignon</p>"
        self.assertEqual(dehtml(html_text, allow_html_chars=True), text)

    def test_special_chars_two(self):
        text = "This \xf6is \xe4 simple text\xfc"
        html_text = "This &ouml;is &auml; simple text&uuml;"
        self.assertEqual(dehtml(html_text), text)



# -*- coding: utf-8 -*-
""""""

from django.core.exceptions import PermissionDenied
from django.contrib.auth.views import redirect_to_login
from django.urls import NoReverseMatch

from coop_cms.moves import MiddlewareMixin, is_authenticated
from coop_cms.utils import get_login_url


class PermissionsMiddleware(MiddlewareMixin):
    """Handle permission"""

    def process_exception(self, request, exception):
        """manage exception"""

        if isinstance(exception, PermissionDenied) and (not is_authenticated(request.user)):
            try:
                login_url = get_login_url()
            except NoReverseMatch:
                login_url = None
            return redirect_to_login(request.path, login_url)



# Generated by Django 2.0.13 on 2020-01-24 13:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0016_auto_20190509_2158'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'ordering': ('ordering',), 'verbose_name': 'document', 'verbose_name_plural': 'documents'},
        ),
        migrations.AlterModelOptions(
            name='image',
            options={'ordering': ('ordering',), 'verbose_name': 'image', 'verbose_name_plural': 'images'},
        ),
    ]



# -*- coding: utf-8 -*-
"""for unit-testing"""

from django.conf.urls import url

from coop_cms.apps.test_app import views


urlpatterns = [
    url(r'^coop-cms-testclass/$', views.TestClassListView.as_view(), name='coop_cms_testapp_list'),
    url(r'^works/(?P<pk>\d+)/cms_edit$', views.TestClassEditView.as_view(), name='coop_cms_testapp_edit'),
    url(r'^works/(?P<pk>\d+)/$', views.TestClassDetailView.as_view(), name='coop_cms_testapp_detail'),
    url(r'^works/cms_edit/$', views.TestClassFormsetEditView.as_view(), name='coop_cms_testapp_formset_edit'),
    url(r'^works/$', views.TestClassFormsetView.as_view(), name='coop_cms_testapp_formset'),
]



# -*- coding: utf-8 -*-
"""coop_bar configuration: add links to balafon"""

from django.urls import reverse
from django.utils.translation import ugettext as _

from coop_bar.utils import make_link

from coop_cms.utils import get_model_name, get_model_app, get_model_label
from .models import Emailing


def newsletters(request, context):
    """back to balafon newsletters"""
    if request and request.user.is_authenticated and request.user.is_staff:
        view_name = 'admin:{0}_{1}_changelist'.format(get_model_app(Emailing), get_model_name(Emailing))
        return make_link(
            reverse(view_name), _('Emailings'), 'table',
            classes=['icon', 'alert_on_click']
        )


def load_commands(coop_bar):
    """load commands"""
    coop_bar.register([
        [newsletters],
    ])



# -*- coding: utf-8 -*-
"""
misc. templatetags
"""

import os.path
import time
import unicodedata

from bs4 import BeautifulSoup

from django import template
from django.conf import settings
from django.template.base import TemplateSyntaxError
from django.template.loader import get_template
from django.utils.safestring import mark_safe

import floppyforms.__future__ as floppyforms

from coop_cms.models import ArticleCategory, Image, Document
from coop_cms.moves import make_context
from coop_cms.settings import get_article_class, logger
from coop_cms.shortcuts import get_article
from coop_cms.utils import dehtml as do_dehtml, slugify
from coop_cms.templatetags.coop_edition import _extract_if_node_args

register = template.Library()


class ArticleLinkNode(template.Node):
    """create an article and returns a link into template"""

    def __init__(self, title, lang):
        self.title = title
        self.lang = lang

    def get_language(self, context):
        """get language from context"""
        request = context.get('request', None)
        lang = "en"
        if request:
            lang = getattr(request, 'LANGUAGE_CODE', lang)
        elif hasattr(settings, 'LANGUAGES'):
            lang = settings.LANGUAGES[0][0]
        elif hasattr(settings, 'LANGUAGE_CODE'):
            lang = settings.LANGUAGE_CODE[:2]
        return lang

    def render(self, context):
        """to html"""
        article_class = get_article_class()
        
        try:
            variable = template.Variable(self.title)
            title = variable.resolve(context)
        except template.VariableDoesNotExist:
            title = self.title.strip("'").strip('"')
        
        slug = slugify(title)
        try:
            if self.lang:
                article = get_article(slug, force_lang=self.lang)
            else:
                # If the language is not defined, we need to get it from the context
                # The Django get_language api doesn't work in template-tag
                article = get_article(slug, current_lang=self.get_language(context))
        except article_class.DoesNotExist:
            try:
                article = get_article(slug, all_langs=True)
            except article_class.DoesNotExist:
                article = article_class.objects.create(slug=slug, title=title)
        return article.get_absolute_url()


@register.tag
def article_link(parser, token):
    """template tag"""
    args = token.split_contents()
    title = args[1]
    lang = args[2] if len(args) > 2 else None
    return ArticleLinkNode(title, lang)


@register.filter
def dehtml(value):
    """
    html to text : Remove all tags
    <p>Hello&nbsp;World<p> --> Hello World
    """
    return do_dehtml(value)


@register.filter
def detagiffy(value):
    """
    html to text: Remove tags but keep html chars
    <p>Hello&nbsp;World<p> --> Hello&nbsp;World
    """
    return mark_safe(do_dehtml(value, allow_html_chars=True))

@register.filter
def sp_rt_lb(value):
    """clean"""
    return value.replace("\n", " ").replace("\r", "")


class NewsletterFriendlyCssNode(template.Node):
    """css in tags attributes"""
    def __init__(self, nodelist_content, css, css_order):
        self.css = css
        self.css_order = css_order
        self.nodelist_content = nodelist_content

    def _style_to_dict(self, style):
        """
        convert a style string ('color: #fff; background: #000') into a dict {'color': ''#fff', 'background': '#000'}
        """
        css_values = [elt for elt in style.strip().split(";") if elt]
        css_values = [elt.split(":") for elt in css_values]
        return dict([(key.strip(), value.strip()) for (key, value) in css_values])

    def _style_to_list(self, style):
        """
        convert a style string ('color: #fff; background: #000') into a list ['color', 'background']
        """
        css_values = [elt for elt in style.strip().split(";") if elt]
        css_values = [elt.split(":") for elt in css_values]
        return [key_and_value[0].strip() for key_and_value in css_values]

    def _dict_to_style(self, style_dict, order_of_items):
        """
        convert a dict {'color': ''#fff', 'background': '#000'} into a style string ('color: #fff; background: #000')
        """
        values = []
        for elt in order_of_items:
            value = style_dict.pop(elt, '')
            if value:
                values.append("{0}: {1}".format(elt, value))
        values.extend(["{0}: {1}".format(key, value) for (key, value) in style_dict.items()])
        if values:
            return "; ".join(values) + ";"
        return ""

    def render(self, context):
        """to html"""
        content = self.nodelist_content.render(context)
        if context.get('by_email', False):
            # avoid string.format issues with curly brackets
            try:
                soup = BeautifulSoup(content, "html.parser")
            except Exception as msg:
                logger.error("HTMLParseError: %s", msg)
                logger.error(content)
                raise

            for tag in self.css_order:
                css = self.css[tag]
                key_and_values = self._style_to_dict(css)
                key_order = self._style_to_list(css)
                for html_tag in soup.select(tag):
                    try:
                        # do not overwrite an inline css value
                        style = html_tag["style"]
                        style_list = self._style_to_list(style)
                        style_dict = self._style_to_dict(style)
                    except KeyError:
                        style_dict = {}
                        style_list = key_order
                    for key in key_order:
                        if key not in style_dict:
                            value = key_and_values[key]
                            style_dict[key] = value
                    # keep items in order
                    html_tag["style"] = self._dict_to_style(style_dict, style_list)

            # Do not prettify : it may cause some display problems
            content = '{0}'.format(soup)  # .prettify(formatter="minimal")

        else:
            style = ""
            for tag in reversed(list(self.css.keys())):
                value = self.css[tag]
                style += "{0} {{ {1} }}\n".format(tag, value)
            content = "<style>\n{0}</style>\n".format(style) + content
        return content


@register.tag
def nlf_css(parser, token):
    """Newsletter friendly CSS"""
    args = token.split_contents()
    css = {}
    css_order = []
    for item in args[1:]:
        tag, value = item.split("=")
        tag, value = tag.strip('"'), value.strip('"')
        css[tag] = value
        css_order.append(tag)
    nodelist = parser.parse(('end_nlf_css',))
    token = parser.next_token()
    return NewsletterFriendlyCssNode(nodelist, css, css_order)


@register.filter
def normalize_utf8_to_ascii(ustr):
    """utf to ascii"""
    try:
        return unicodedata.normalize('NFKD', ustr).encode('ascii', 'ignore')
    except TypeError:
        return ustr
    

@register.filter(name='is_checkbox')
def is_checkbox(field):
    """is checkbox"""
    field = getattr(field, 'field', field) # get the field attribute of the field or the field itself
    if hasattr(field, 'widget'):
        return field.widget.__class__.__name__ == floppyforms.CheckboxInput().__class__.__name__
    return False


@register.filter
def index(seq, index_val):
    """get index"""
    try:
        return seq[index_val]
    except IndexError:
        return None


class CoopCategoryNode(template.Node):
    """get category in template context"""
    def __init__(self, cat_slug, var_name):
        self.category = None
        cat = cat_slug.strip("'").strip('"')
        self.cat_var, self.cat = None, None
        if cat_slug == cat:
            self.cat_var = template.Variable(cat)
        else:
            self.cat = cat
        self.var_name = var_name

    def render(self, context):
        """to html"""
        if self.cat_var:
            self.cat = self.cat_var.resolve(context)
        try:
            slug = slugify(self.cat)
            self.category = ArticleCategory.objects.get(slug=slug)
        except ArticleCategory.DoesNotExist:
            self.category = ArticleCategory.objects.create(name=self.cat)
        context.dicts[0][self.var_name] = self.category
        return ""


@register.tag
def coop_category(parser, token):
    """get category in template context"""
    args = token.split_contents()
    cat_slug = args[1]
    var_name = args[2]
    return CoopCategoryNode(cat_slug, var_name)


@register.filter
def basename(fullname):
    """os.path.basename"""
    return os.path.basename(fullname)


@register.filter
def get_parts(list_of_objs, number_of_parts):
    """slice"""
    nb_objs = len(list_of_objs)
    nb_by_part, extra_nb = nb_objs // number_of_parts, nb_objs % number_of_parts
    parts = []
    stop_index = 0
    for which_part in range(number_of_parts):
        start_index = 0 if (stop_index == 0) else stop_index
        stop_index = start_index + nb_by_part + (1 if (which_part < extra_nb) else 0)
        parts.append(list(list_of_objs[start_index:stop_index]))
    return parts


@register.filter
def get_part(list_of_objs, partionning):
    """slices"""
    which_part, number_of_parts = [int(x) for x in partionning.split("/")]
    parts = get_parts(list_of_objs, number_of_parts)
    return parts[which_part-1]


@register.filter
def group_in_sublists(list_of_objs, subslist_size):
    """group_in_sublists([1, 2, 3, 4], 2) --> [[1, 2], [3, 4]]"""
    return [list_of_objs[i:i+subslist_size] for i in range(0, len(list_of_objs), subslist_size)]


class MediaListNode(template.Node):
    def __init__(self, model_class, filter_name, var_name):
        self.model_class = model_class
        stripped_filter_name = filter_name.strip("'").strip('"')
        self.filter_var, self.filter_value = None, None
        if stripped_filter_name == filter_name:
            self.filter_var = template.Variable(filter_name)
        else:
            self.filter_value = stripped_filter_name
        self.var_name = var_name

    def render(self, context):
        if self.filter_var:
            self.filter_value = self.filter_var.resolve(context)
        images = self.model_class.objects.filter(filters__name=self.filter_value).order_by("ordering", "-created")
        context.dicts[0][self.var_name] = images
        return ""


@register.tag
def coop_image_list(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        filter_name = args[1]
        # as_name = args[2]
        var_name = args[3]
    except IndexError:
        raise TemplateSyntaxError("coop_image_list: usage --> {% coop_image_list 'filter_name' as var_name %}")
    return MediaListNode(Image, filter_name, var_name)


DEFAULT_ACCEPT_COOKIE_MESSAGE_TEMPLATE = 'coop_cms/_accept_cookies_message.html'


class ShowAcceptCookieMessageNode(template.Node):
    """accept cookie message"""
    def __init__(self, template_name):
        self.template_name = template_name or DEFAULT_ACCEPT_COOKIE_MESSAGE_TEMPLATE
        super(ShowAcceptCookieMessageNode, self).__init__()

    def render(self, context):
        """to html"""
        request = context.get('request', None)
        if request and request.session.get('accept_cookies', None) is None:
            template_ = get_template(self.template_name)
            return template_.render(make_context(request, {}))
        else:
            return ""


@register.tag
def show_accept_cookie_message(parser, token):
    """show accept cookie message"""
    args = token.split_contents()
    if len(args) > 1:
        template_name = args[1].strip('"').strip('"')
    else:
        template_name = ""
    return ShowAcceptCookieMessageNode(template_name)


class IfAcceptCookiesNode(template.Node):
    """Do something if user accept cookies"""

    def __init__(self, nodelist_true, nodelist_false):
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false

    def __iter__(self):
        for node in self.nodelist_true:
            yield node
        for node in self.nodelist_false:
            yield node

    def _check_condition(self, request):
        """True only if accept cookies is explicitly Yes"""
        return request.session.get('accept_cookies', False)

    def render(self, context):
        """to html"""
        request = context.get('request', None)
        if request and self._check_condition(request):
            return self.nodelist_true.render(context)
        else:
            return self.nodelist_false.render(context)


@register.tag
def if_accept_cookies(parser, token):
    """Do something if edition mode"""
    nodelist_true, nodelist_false = _extract_if_node_args(parser, token)
    return IfAcceptCookiesNode(nodelist_true, nodelist_false)


@register.filter
def open_tag_if(index_, args):
    """open_tag if condition"""
    tag, nb_per_block = args.split("/")
    nb_per_block = int(nb_per_block)
    return mark_safe("<{0}>".format(tag) if (index_ % nb_per_block) == 0 else "")


@register.filter
def close_tag_if(index_, args):
    """close_tag if condition"""
    tag, nb_per_block = args.split("/")
    nb_per_block = int(nb_per_block)
    return mark_safe("</{0}>".format(tag) if (index_ % nb_per_block) == nb_per_block else "")


@register.filter
def find_css(value, css_class):
    """open_tag if condition"""
    if css_class in value.split(" "):
        return True
    return False


class VersionedStaticFileNode(template.Node):
    """return static path with ?v=dateoflastchange"""
    def __init__(self, static_path):
        self.static_path = static_path.strip("'").strip('"')
        self.is_string = self.static_path == static_path

    def render(self, context):
        if settings.DEBUG:
            version = time.time()
        else:
            static_file_path = os.path.join(settings.STATIC_ROOT, self.static_path)
            try:
                # last Modification of the file
                version = os.path.getmtime(static_file_path)
            except OSError:
                version = 'x'
        return mark_safe("{0}{1}?v={2}".format(settings.STATIC_URL, self.static_path, version))


@register.tag
def versioned_static_file(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        static_path = args[1]
    except IndexError:
        raise TemplateSyntaxError("coop_image_list: usage --> {% versioned_static_file 'static_path' %}")
    return VersionedStaticFileNode(static_path)


@register.tag
def coop_docs_list(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        filter_name = args[1]
        # as_name = args[2]
        var_name = args[3]
    except IndexError:
        raise TemplateSyntaxError("coop_docs_list: usage --> {% coop_docs_list 'filter_name' as var_name %}")
    return MediaListNode(Document, filter_name, var_name)


@register.filter
def reduced_page_range(page_obj, max_num=10):
    nb_pages = len(page_obj.paginator.page_range)

    if nb_pages > max_num:
        current_page = page_obj.number

        start_value = max(1, current_page - 2)
        pages = list(range(start_value, current_page))

        pages += [current_page]

        end_value = min(nb_pages, current_page + 2)
        pages += list(range(current_page + 1, end_value + 1))

        if 1 not in pages:
            # If not in the list add first page
            extra_pages = [1, ]
            if pages[0] != 2:
                # Add separator if there is a gap between numbers
                extra_pages += [0, ]
            pages = extra_pages + pages

        if nb_pages not in pages:
            # If not in the list add separator and last page
            extra_pages = [nb_pages, ]
            if pages[-1] != nb_pages - 1:
                extra_pages = [0, ] + extra_pages
            pages += extra_pages

        return pages
    else:
        return page_obj.paginator.page_range


@register.filter
def unicode_slugify(value):
    """open_tag if condition"""
    return slugify(value)



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from django_registration.backends.activation.views import RegistrationView, ActivationView

from coop_cms.apps.email_auth.registration_backend.forms import RegistrationFormUniqueEmailAndTermsOfService


def notify_event(emails_to_notify, subject, template_name, extra_context):
    """notify an event to site managers"""
    if emails_to_notify:
        site = Site.objects.get_current()
        full_subject = "{0} - {1}".format(site, subject)
        the_template = get_template(template_name)
        context_dict = {'site': site}
        if extra_context:
            context_dict.update(extra_context)
        message = the_template.render(context_dict)
        send_mail(full_subject, message, settings.DEFAULT_FROM_EMAIL, list(emails_to_notify))


class EmailRegistrationView(RegistrationView):
    """register with email address"""
    form_class = RegistrationFormUniqueEmailAndTermsOfService

    def register(self, *args, **kwargs):
        new_user = super(EmailRegistrationView, self).register(*args, **kwargs)
        self.notify(new_user)
        return new_user

    def notify(self, new_user):
        """notify a list of people when a new account is creted"""
        emails_to_notify = getattr(settings, 'COOP_CMS_ACCOUNT_REGISTRATION_NOTIFICATION_EMAILS', None)
        notify_event(
            emails_to_notify,
            _("Account created"),
            'email_auth/registration_notification.txt',
            {'user': new_user, }
        )


class EmailActivationView(ActivationView):
    """activate account after user clicked on tokenized link"""

    def activate(self, *args, **kwargs):
        activated_user = super(EmailActivationView, self).activate(*args, **kwargs)
        self.notify(activated_user)
        return activated_user

    def notify(self, activated_user):
        """notify a list of people when a new account is activated"""
        emails_to_notify = getattr(settings, 'COOP_CMS_ACCOUNT_ACTION_NOTIFICATION_EMAILS', None)
        notify_event(
            emails_to_notify,
            _("Account activated"),
            'email_auth/activation_notification.txt',
            {'user': activated_user, }
        )



# -*- coding: utf-8 -*-

from django.conf.urls import url
from django.views.generic.base import TemplateView
from coop_cms.apps.email_auth.registration_backend.views import EmailRegistrationView, EmailActivationView

urlpatterns = [
    url(
        r'^activate/complete/$',
        TemplateView.as_view(template_name='django_registration/activation_complete.html'),
        name='django_registration_activation_complete'
    ),
    # Activation keys get matched by \w+ instead of the more specific
    # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
    # that way it can return a sensible "invalid key" message instead of a
    # confusing 404.
    url(
        r'^activate/(?P<activation_key>.+)/$',
        EmailActivationView.as_view(),
        name='registration_activate'
    ),
    url(
        r'^register/$',
        EmailRegistrationView.as_view(),
        name='registration_register'
    ),
    url(
        r'^register/complete/$',
        TemplateView.as_view(template_name='django_registration/registration_complete.html'),
        name='django_registration_complete'
    ),
    url(
        r'^register/closed/$',
        TemplateView.as_view(template_name='django_registration/registration_closed.html'),
        name='django_registration_disallowed'
    ),
]



# -*- coding: utf-8 -*-
"""modeltranslation settings"""

from modeltranslation.translator import translator, TranslationOptions  # pylint: disable=F0401

from coop_cms.models import (
    Alias, ArticleCategory, Fragment, NavNode, Newsletter, PieceOfHtml, SiteSettings, validate_slug, Link
)
from coop_cms.settings import get_eastern_languages


class PieceOfHtmlTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('content',)
translator.register(PieceOfHtml, PieceOfHtmlTranslationOptions)


class FragmentTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('content',)
translator.register(Fragment, FragmentTranslationOptions)


class NavNodeTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('label',)
translator.register(NavNode, NavNodeTranslationOptions)


class ArticleCategoryTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('name',)
translator.register(ArticleCategory, ArticleCategoryTranslationOptions)


class SiteSettingsTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('homepage_url', 'homepage_article',)
translator.register(SiteSettings, SiteSettingsTranslationOptions)


class AliasTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('path', 'redirect_url',)
translator.register(Alias, AliasTranslationOptions)


class NewsletterTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('subject', 'content',)
translator.register(Newsletter, NewsletterTranslationOptions)


class LinkTranslationOptions(TranslationOptions):
    """translation"""
    fields = ('url',)
translator.register(Link, LinkTranslationOptions)


class BaseArticleTranslationOptions(TranslationOptions):
    """
    Handle eastern languages (russian, chinese, japanese...) and slugs : Do not validate in this case
    Inherit the ArticleTranslationOptions from this base class
    """

    class SlugValidator(object):

        def __init__(self, is_eastern_language):
            self.is_eastern_language = is_eastern_language

        def __call__(self, value):
            if not self.is_eastern_language:
                validate_slug(value)

    def add_translation_field(self, field, translation_field):
        """
        Add a new translation field to both fields dicts.
        """
        # Patch the slug field in order not to validate slug if easten
        if field == 'slug':  # and translation_field.language in get_eastern_languages():
            if validate_slug in translation_field.validators:
                translation_field.validators.remove(validate_slug)
            translation_field.validators = [
                BaseArticleTranslationOptions.SlugValidator(translation_field.language in get_eastern_languages())
            ] + translation_field.validators
        super(BaseArticleTranslationOptions, self).add_translation_field(field, translation_field)




# -*- coding: utf-8 -*-
"""newsleters"""

from datetime import datetime
import sys
from tempfile import NamedTemporaryFile

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.contrib.sites.models import Site
from django.core.exceptions import PermissionDenied, ValidationError
from django.http import HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.utils.encoding import smart_text
from django.utils.translation import ugettext as _
from django.views.generic import View

from colorbox.decorators import popup_redirect

from coop_cms.forms.newsletters import NewsletterSchedulingForm, NewsletterTemplateForm, NewsletterHandleRecipients
from coop_cms import models
from coop_cms.generic_views import EditableObjectView
from coop_cms.logger import logger
from coop_cms.optionals import convert_to_pdf, make_absolute_paths, PDFResponse
from coop_cms.settings import get_newsletter_form, get_newsletter_settings_form
from coop_cms.utils import send_newsletter, slugify


@login_required
@popup_redirect
def newsletter_settings(request, newsletter_id=None):
    """edit or created newsletter"""

    if newsletter_id:
        newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)
    else:
        newsletter = None

    form_class = get_newsletter_settings_form()

    if request.method == "POST":
        form = form_class(request.user, request.POST, instance=newsletter)
        if form.is_valid():
            newsletter = form.save()
            return HttpResponseRedirect(newsletter.get_absolute_url())
    else:
        form = form_class(request.user, instance=newsletter)

    return render(
        request,
        'coop_cms/popup_newsletter_settings.html',
        locals()
    )


@login_required
@popup_redirect
def change_newsletter_template(request, newsletter_id):
    """change newsletter template"""
    newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)

    if not request.user.has_perm('can_edit_newsletter', newsletter):
        raise PermissionDenied

    if request.method == "POST":
        form = NewsletterTemplateForm(newsletter, request.user, request.POST)
        if form.is_valid():
            newsletter.template = form.cleaned_data['template']
            newsletter.save()
            return HttpResponseRedirect(newsletter.get_edit_url())
    else:
        form = NewsletterTemplateForm(newsletter, request.user)

    return render(
        request,
        'coop_cms/popup_change_newsletter_template.html',
        {'form': form, 'newsletter': newsletter}
    )


@login_required
@popup_redirect
def test_newsletter(request, newsletter_id):
    """test newsletter"""
    context = {}
    newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)
    context["newsletter"] = newsletter

    if not request.user.has_perm('can_edit_newsletter', newsletter):
        raise PermissionDenied

    if request.method == "POST":
        form = NewsletterHandleRecipients(data=request.POST)
        if form.is_valid():
            emails = form.cleaned_data["emails"]
            additional_email1 = form.cleaned_data["additional_email1"]
            additional_email2 = form.cleaned_data["additional_email2"]
            recipients = emails or []
            if additional_email1:
                recipients.append(additional_email1)
            if additional_email2 and additional_email1 != additional_email2:
                recipients.append(additional_email2)
            try:
                nb_sent = send_newsletter(newsletter, recipients)
                messages.add_message(
                    request, messages.SUCCESS,
                    _("The test email has been sent to {0} addresses: {1}").format(nb_sent, recipients)
                )
                return HttpResponseRedirect(newsletter.get_absolute_url())

            except Exception:
                messages.add_message(request, messages.ERROR, _("An error occurred! Please contact your support."))
                logger.error(
                    'Internal Server Error: {0}'.format(request.path),
                    exc_info=sys.exc_info,
                    extra={
                        'status_code': 500,
                        'request': request
                    }
                )
                return HttpResponseRedirect(newsletter.get_absolute_url())
    else:
        form = NewsletterHandleRecipients()

    context["form"] = form
    return render(request, 'coop_cms/popup_test_newsletter.html', context)


@login_required
@popup_redirect
def schedule_newsletter_sending(request, newsletter_id):
    """schedule sending"""
    newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)
    instance = models.NewsletterSending(newsletter=newsletter)

    if request.method == "POST":
        form = NewsletterSchedulingForm(request.POST, instance=instance)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(newsletter.get_edit_url())
    else:
        form = NewsletterSchedulingForm(instance=instance, initial={'scheduling_dt': datetime.now()})

    return render(
        request,
        'coop_cms/popup_schedule_newsletter_sending.html',
        {'newsletter': newsletter, 'form': form}
    )


class NewsletterView(EditableObjectView):
    """newsletter view for edition"""
    model = models.Newsletter
    form_class = get_newsletter_form()
    field_lookup = "id"
    varname = "newsletter"
    editable = True

    def can_view_object(self):
        if self.object.is_public:
            return True
        return super(NewsletterView, self).can_edit_object()

    def get_context_data(self):
        """context"""
        context_data = super(NewsletterView, self).get_context_data()
        context_data.update({
            'title': self.object.subject,
            'by_email': self.request.GET.get('by_email', False),
            'editable': self.editable,
            'current_site': self.object.site or Site.objects.get_current(),
            'COOP_CMS_SITE_PREFIX': settings.COOP_CMS_SITE_PREFIX,
        })
        return context_data

    def after_save(self, article):
        """after save"""
        pass

    def get_template(self):
        """get template"""
        return self.object.get_template_name()


class NewsletterPdfView(View):
    """convert the newsletter to pdf"""

    def get(self, request, *args, **kwargs):
        """handle GET request : convert newsletter to pdf"""

        newsletter = get_object_or_404(models.Newsletter, id=self.kwargs['id'])

        # Call the newsletter view as a regular function
        newsletter_view = NewsletterView.as_view(editable=False)
        response = newsletter_view(request, id=newsletter.id)

        # Save the Html into a temporary file
        temp_file = self.save_to_temporary_file(response.content)

        # convert this file to pdf
        pdf_content = convert_to_pdf(
            filename=temp_file.name,
            header_filename=None,
            footer_filename=None,
            cmd_options={}
        )

        # Generate name of the pdf file
        filename = 'newsletter_{0}.pdf'.format(slugify(newsletter.subject))

        # returns PDF response
        return PDFResponse(pdf_content, show_content_in_browser=False, filename=filename)

    def save_to_temporary_file(self, content):
        """
        put content into a temporary file
        Inspired by django-wkhtmltopdf
        """

        # Turn path to absolute
        content = smart_text(content)
        content = make_absolute_paths(content)

        try:
            # Python3 has 'buffering' arg instead of 'bufsize'
            temp_file = NamedTemporaryFile(
                mode='w+b', buffering=-1, suffix='.html', prefix='tmp', dir=None, delete=True)
        except TypeError:
            temp_file = NamedTemporaryFile(
                mode='w+b', bufsize=-1, suffix='.html', prefix='tmp', dir=None, delete=True
            )

        try:
            temp_file.write(content.encode('utf-8'))
            temp_file.flush()
            return temp_file
        except:
            # Clean-up tempfile if an Exception is raised.
            temp_file.close()
            raise



# -*- coding: utf-8 -*-
"""test email sending"""

from coop_cms.tests import BaseTestCase

from ..utils import force_line_max_length


class MaxLineLengthTest(BaseTestCase):
    """Check that we can control the max length of line"""

    def test_force_line_max_length(self):
        """it should not have a line more than 400 chars"""

        in_lines = [
            'This is a messsage',
            "123456789 " * 50,
            "That's all folks!"
        ]
        text = "\n".join(in_lines)

        out_text = force_line_max_length(text, max_length_per_line=400, dont_cut_in_quotes=True)
        out_lines = out_text.split("\n")

        self.assertEqual(len(out_lines), 4)
        self.assertEqual(out_lines[0], in_lines[0])
        self.assertEqual(out_lines[1], "123456789 " * 40 + "123456789")
        self.assertEqual(out_lines[2], "123456789 " * 8 + "123456789")
        self.assertEqual(out_lines[3], in_lines[2])

    def test_force_line_max_length_in_p(self):
        """it should not have a line more than 400 chars"""

        content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>
            <p>{0}</p>
            </html>
        """.format("123456789 " * 50)

        expected_content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>\n<p>{0}\n{1}</p>
            </html>
        """.format(
            "123456789 " * 39 + "123456789",
            "123456789 " * 10
        )

        result = force_line_max_length(content, max_length_per_line=400, dont_cut_in_quotes=False)
        self.assertEqual(expected_content, result)

    def test_force_line_max_length_a1(self):
        """it should not have a line more than 400 chars"""

        content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>
            <p>{0}<a href="{1}">This is a link</a></p>
            </html>
        """.format("123456789 " * 36, "this-is-a-link-dont-cut-in-quotes")

        expected_content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>\n<p>{0}<a href="{1}">This\nis a link</a></p>
            </html>
        """.format("123456789 " * 36, "this-is-a-link-dont-cut-in-quotes")

        result = force_line_max_length(content,  max_length_per_line=400, dont_cut_in_quotes=False)

        self.assertEqual(expected_content, result)

    def test_force_line_max_length_a2(self):
        """it should not have a line more than 400 chars"""

        content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>
            <p>{0}<a class="abcdefgh ijklmnop qrstuvwxyz" href="{1}">This is a link</a></p>
            </html>
        """.format("123456789 " * 38, "this-is-a-link-dont-cut-in-quotes")

        expected_content = """
            <html>
            <body>
            </body>
            <h1>Title</h1>
            <p>
                Paragraph1
                Ok
            </p>
            <p>Paragraph 2</p>\n<p>{0}<a class="abcdefgh ijklmnop qrstuvwxyz"\nhref="{1}">This is a link</a></p>
            </html>
        """.format("123456789 " * 38, "this-is-a-link-dont-cut-in-quotes")

        result = force_line_max_length(content, max_length_per_line=400, dont_cut_in_quotes=False)
        self.assertEqual(expected_content, result)
